@@ -33,7 +33,10 @@
 #endif
 
 #include <stdlib.h>
+#include <string.h>
+#include <errno.h>
 #ifdef _MSC_VER
+#include <windows.h>
 #include <direct.h>
 #include <io.h>
 #else
@@ -41,36 +44,31 @@
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <dirent.h>
 #endif
-#include <errno.h>
 
-#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
 #include <algorithm>
+#include <map>
 #include <vector>
 #include <string>
 #include <iomanip>
-#ifdef _MSC_VER
-#include <typeinfo.h>
 #include <iostream>
 #include <fstream>
 #include <strstream>
+#ifdef _MSC_VER
+#include <typeinfo.h>
 #else
 #include <typeinfo>
-#include <iostream.h>
-#include <fstream.h>
-#include <strstream.h>
-#endif
-
-#ifndef _MSC_VER
-#include <dirent.h>
 #endif
 
 using namespace std;
 
-
+#include "Options.h"
+#include "Profile.h"
+#include "TargetD64.h"
 #include "Exception.h"
 #include "Image.h"
 #include "DiskImage.h"
@@ -83,21 +81,40 @@ using namespace std;
 
 string CFArchive::ms_tmpdir;
 string CFArchive::ms_emuArchiveBasename;
-int CFArchive::ms_emuArchiveCount = 0;
+unsigned int CFArchive::ms_emuArchiveCount = 0;
 string CFArchive::ms_emuImageDir;
 string CFArchive::ms_miscDir;
 bool CFArchive::ms_bEmuImageCleanup = true;
 bool CFArchive::ms_bEmuImageNameDueOriginal = false;
-bool CFArchive::ms_bUseExternalHelper = false;
+bool CFArchive::ms_bCleanup = true;
 
 vector<CFArchive *> CFArchive::ms_emuArchives;
 
+
+//a little POSIX conformance for M$
+#ifdef _MSC_VER
+
+//provide UNIX scan directory functions
+typedef HANDLE DIR;
+
+struct dirent
+{
+	char d_name[MAX_PATH];
+};
+
+static DIR *opendir(const char *name);
+static int closedir(DIR *dir);
+static struct dirent *readdir(DIR *dir);
+
+#endif
+
+
 string& CFArchive::BuildHostConformFilename(const string& filename)
 {
 	static string retFilename;
 	
 	retFilename = filename;
-	for (int i = 0; i < filename.size(); i++)
+	for (string::size_type i = 0; i < filename.size(); i++)
 	{
 		//replace non usable characters by _
 		if ((retFilename[i] == '\'')
@@ -109,6 +126,40 @@ string& CFArchive::BuildHostConformFilename(const string& filename)
 	return retFilename;
 }
 
+
+#ifdef _MSC_VER
+
+//commands are executed in batchfiles for M$
+//% characters have to be auoted by percent characters in batchfiles
+static const string& QuotePercentCharacter(const string& in)
+{
+	static string retString;
+
+	retString = in;
+
+	//ATTENTION: % character in M$ batchfiles must be % quoted (%%)
+	string::size_type pos = 0;
+	while ((pos = retString.find("%", pos)) != string::npos)
+	{
+		//from the left to the percent (included)
+		string leftEqualPercent = retString.substr(0, pos + 1);
+		//from the percent (included) to the right
+		string rightEqualPercent = retString.substr(pos
+			, retString.size() - pos);
+		//this is the percent doubling
+		retString = leftEqualPercent + rightEqualPercent;
+		//make sure that originally found percent and appended
+		//percent are not found
+		pos += 2;
+		if (pos >= retString.size())
+			pos = string::npos;
+	}
+	return retString;
+}
+
+#endif
+
+
 //For Linux:
 //replace %s<index> with replacement
 //For Windows:
@@ -119,9 +170,18 @@ string& CFArchive::BuildHostConformFilename(const string& filename)
 //a prefix of "basename " for P3 marks that basename of filename is wanted
 //this is needed to get a short and long filename version for Win32
 //otherwise we could have created the basename before calling
+
+#ifdef _MSC_VER
+//we have batchfile execution - quote % to %%
+#define REPLACEMENT(a) (QuotePercentCharacter(a))
+#else
+#define REPLACEMENT(a) (a)
+#endif
+
 static void ReplaceHelperFunction(string& command, char index, const string& replacement)
+	throw (CFException) //e.g. indirect via SplitPathname
 {
-	int pos = 0;
+	string::size_type pos = 0;
 	string filenameToBuildBasenameFrom;
 
 	//check if somebody has requested a filename's basename
@@ -138,7 +198,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 		{
 			//basename of filename requested
 			string directory, filename, extension;
-			SplitPathname(filenameToBuildBasenameFrom
+			::SplitPathname(filenameToBuildBasenameFrom
 				, directory, filename, extension);
 			tmp = command.substr(0, pos) + filename + extension;
 		}
@@ -164,7 +224,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 			{
 				//basename of filename requested
 				string directory, filename, extension;
-				SplitPathname(CFArchive::ConvertFilenameUnixToWin(filenameToBuildBasenameFrom)
+				::SplitPathname(CFArchive::ConvertFilenameUnixToWin(filenameToBuildBasenameFrom)
 					, directory, filename, extension);
 				tmp = filename + extension;
 			}
@@ -173,7 +233,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 				tmp = CFArchive::ConvertFilenameUnixToWin(replacement);
 			}
 			//add non substituted command and search on
-			command = command.substr(0, pos) + tmp 
+			command = command.substr(0, pos) + REPLACEMENT(tmp)
 				+ command.substr(pos + 3, command.size() - pos - 3);
 			//do not consider substitution for replacement
 			pos += tmp.size();
@@ -189,7 +249,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 				{
 					//basename of filename requested
 					string directory, filename, extension;
-					SplitPathname(CFArchive::ConvertFilenameUnixToWinShort(filenameToBuildBasenameFrom)
+					::SplitPathname(CFArchive::ConvertFilenameUnixToWinShort(filenameToBuildBasenameFrom)
 						, directory, filename, extension);
 					tmp = filename + extension;
 				}
@@ -198,7 +258,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 					tmp = CFArchive::ConvertFilenameUnixToWinShort(replacement);
 				}
 				//add non substituted command and search on
-				command = command.substr(0, pos) + tmp 
+				command = command.substr(0, pos) + REPLACEMENT(tmp)
 					+ command.substr(pos + 3, command.size() - pos - 3);
 				//do not consider substitution for replacement
 				pos += tmp.size();
@@ -228,7 +288,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 					}
 					string tmp = command.substr(0, pos) + drive;
 					//add non substituted command and search on
-					command = tmp + command.substr(pos + 3, command.size() - pos - 3);
+					command = REPLACEMENT(tmp) + command.substr(pos + 3, command.size() - pos - 3);
 					//do not consider substitution for replacement
 					pos = ((tmp.size() == 0) ? 1 : tmp.size()) - 1;
 				}
@@ -247,7 +307,7 @@ static void ReplaceHelperFunction(string& command, char index, const string& rep
 
 static void BuildAbsoluteDirectoryPath(string &dirPath) throw (CFException)
 {
-	TRBEGIN("GetAbsolutePathOfDirectory");
+	TRBEGIN("BuildAbsoluteDirectoryPath");
 	TRACE(<< "Dirpath: " << dirPath);
 	char *tmpWd = getcwd(NULL, 0); //allocs memory!
 	if (tmpWd == NULL)
@@ -287,11 +347,41 @@ static void BuildAbsoluteDirectoryPath(string &dirPath) throw (CFException)
 }
 
 
-void CFArchive::EvaluateEnvironmentVariables(void)
-	throw (CFException)
+void CFArchive::ReadSettings(void) throw (CFException)
 {
-	TRBEGIN("CFArchive::EvaluateEnvironmentVariables");
+	TRBEGIN("CFArchive::ReadSettings");
 
+	string value;
+	//-----------------------------------------------------------
+	//Profile settings
+	//-----------------------------------------------------------
+	const CFProfile *pProfile = CFProfile::GetAppProfile();
+	ASSERT(pProfile != NULL);
+	bool flag;
+	if (pProfile->ReadProfileValue(PRO_OPT_NOCLEANUP.first, PRO_OPT_NOCLEANUP.second, flag)
+		&& flag)
+	{
+		ms_bCleanup = false;
+	}
+	if (pProfile->ReadProfileValue(PRO_OPT_KEEP.first, PRO_OPT_KEEP.second, flag)
+		&& flag)
+	{
+		ms_bEmuImageNameDueOriginal = true;
+	}
+	string tmpdir;
+	(void)pProfile->ReadProfileValue(PRO_ENV_TMPDIR.first, PRO_ENV_TMPDIR.second, tmpdir);
+
+	//-----------------------------------------------------------
+	//Command line options
+	//-----------------------------------------------------------
+	const CFOptions *pOpt = CFOptions::GetAppOptions();
+	ASSERT(pOpt != NULL);
+	ms_bCleanup &= !pOpt->QueryOption(OPT_NO_CLEANUP.first, value);	//no cleanup
+	ms_bEmuImageNameDueOriginal |= pOpt->QueryOption(OPT_KEEP_FILENAME.first, value);
+
+	//-----------------------------------------------------------
+	//Environment variables
+	//-----------------------------------------------------------
 	//either use environment variable TMPDIR or /tmp
 	static char* env = ::getenv("TMPDIR");
 #ifdef _MSC_VER
@@ -303,10 +393,19 @@ void CFArchive::EvaluateEnvironmentVariables(void)
 			env = ::getenv("TEMP");
 		}
 	}
-	string tmpdir = (env) ? env : "c:\\temp";
-#else
-	string tmpdir = (env) ? env : "/tmp";
 #endif
+	if (env != NULL)
+	{
+		tmpdir = env;
+	}
+	else if (tmpdir.empty())
+	{
+#ifdef _MSC_VER
+		tmpdir = "c:\\temp";
+#else
+		tmpdir = "/tmp";
+#endif
+	}
 
 	//now eval the absolute path - ATTENTION: must be absolute
 	::BuildAbsoluteDirectoryPath(tmpdir);
@@ -314,31 +413,10 @@ void CFArchive::EvaluateEnvironmentVariables(void)
 	//now that the temp dir is determined set it
 	ms_tmpdir = tmpdir;
 
-	if ((env = ::getenv("TD64_ZIP_CMD")) != NULL)
-	{
-		TRACE(<< "Zip extract command set to: " << env);
-		CFZipArchive::SetExtractCommand(env);
-	}
-	if ((env = ::getenv("TD64_LHA_CMD")) != NULL)
-	{
-		TRACE(<< "Lha extract command set to: " << env);
-		CFLhaArchive::SetExtractCommand(env);
-	}
-	if ((env = ::getenv("TD64_GZ_CMD")) != NULL)
-	{
-		TRACE(<< "Gz extract command set to: " << env);
-		CFGzipArchive::SetExtractCommand(env);
-	}
-	if ((env = ::getenv("TD64_LNX_CMD")) != NULL)
-	{
-		TRACE(<< "Lnx extract command set to: " << env);
-		CFLnxArchive::SetExtractCommand(env);
-	}
-	if ((env = ::getenv("TD64_C64ZIP_CMD")) != NULL)
-	{
-		TRACE(<< "C64zip extract command set to: " << env);
-		CFZipC64Archive::SetExtractCommand(env);
-	}
+	//-----------------------------------------------------------
+	//Generic archives from environment and profile
+	//-----------------------------------------------------------
+
 	//check for generic archive extraction commands
 	//Search for pairs of
 	//TD64_GENERIC_CMDx and TD64_GENERIC_EXTENSIONx
@@ -378,6 +456,39 @@ void CFArchive::EvaluateEnvironmentVariables(void)
 		else
 			i = -1;
 	}
+
+	//now read generic archive definitions from profile
+
+	vector<string> genericArchives;
+	pProfile->ReadProfileValue(PRO_GENERIC_ARCHIVES.first, PRO_GENERIC_ARCHIVES.second, genericArchives);
+	for (vector<string>::size_type j = 0; j < genericArchives.size(); j++)
+	{
+		string strExt, strCmd;
+		string::size_type nSeparator = genericArchives[j].find(',');
+		if (nSeparator != string::npos)
+		{
+			strExt = genericArchives[j].substr(0, nSeparator);
+			strCmd = genericArchives[j].substr(nSeparator + 1, genericArchives[j].size() - nSeparator - 1);
+			TRACE(<< "Generic archive extension from profile: " << ": " << strExt);
+			TRACE(<< "Generic archive command from profile: " << ": " << strCmd);
+			//user defined generic archive encountered - queue it
+			CFGenericArchive userDefinedGenericArchiv(
+				"", strCmd, strExt, 0);
+			//mind that vector take a copy of the auto class
+			//variable userDefinedGenericArchiv
+			CFGenericArchive::ms_genericArchives
+				.push_back(userDefinedGenericArchiv);
+		}
+		else
+		{
+			string param1 = "Syntax error in generic archive definition: "
+				+ genericArchives[j];
+			exc = CFException(CFException::PROFILE_ERROR,
+				__LINE__, __FILE__, param1, "");
+			exc.WriteOutExceptionWithWarningHeader(cerr);
+		}
+	}
+
 	TRreturn;
 }
 
@@ -394,7 +505,7 @@ string& CFArchive::ReplacePlaceHoldersByStrings(
 	, const string& s2
 	, const string& s3 /*= ""*/
 	, const string& s4 /*= ""*/
-	, const string& s5 /*= ""*/)
+	, const string& s5 /*= ""*/) throw (CFException)
 {	
 	static string retString;
 
@@ -415,9 +526,11 @@ string& CFArchive::ReplacePlaceHoldersByStrings(
 }
 
 
-CFArchive::CFArchive() : m_state(CREATED), m_blockSize(0)
+CFArchive::CFArchive() : m_pParentArchive(NULL)
+	, m_state(CREATED)
 	, m_final(false)
-	, m_pParentArchive(NULL), m_pDiskImage(NULL)
+	, m_blockSize(0)
+	, m_pDiskImage(NULL)
 	//-1 is greatest unsigned int
 	, m_nRequestOrderIndex((unsigned int)-1)
 {
@@ -426,13 +539,15 @@ CFArchive::CFArchive() : m_state(CREATED), m_blockSize(0)
 }
 
 
-CFArchive::CFArchive(const string& pathname, const int blockSize /*= 0*/
-	, const string& extractCmd /*= const string("")*/)
-	: m_state(CREATED), m_blockSize(blockSize)
+//SetPathComponents may throw exception
+CFArchive::CFArchive(const string& pathname, const unsigned int blockSize /*= 0*/)
+	throw (CFException)
+	: m_pParentArchive(NULL)
+	, m_state(CREATED)
 	, m_final(false)
-	, m_extractCommand(extractCmd)
 	, m_workdir(ms_tmpdir)
-	, m_pParentArchive(NULL), m_pDiskImage(NULL)
+	, m_blockSize(blockSize)
+	, m_pDiskImage(NULL)
 	//-1 is greatest unsigned int
 	, m_nRequestOrderIndex((unsigned int)-1)
 {
@@ -450,14 +565,14 @@ CFArchive::~CFArchive()
 	//ATTENTION:
 	//the following assumes ALL archives created by CreateSubClassObject() or
 	//at least a new operator.
-	for (int i = 0; i < m_files.size(); i++)
+	for (vector<CFArchive *>::size_type i = 0; i < m_files.size(); i++)
 	{
 		delete m_files[i];
 	}
 	//for the ROOT archive cleanup the emulator archives
 	if (m_state == ROOT)
 	{
-		for (int j = 0; j < ms_emuArchives.size(); j++)
+		for (vector<CFArchive *>::size_type j = 0; j < ms_emuArchives.size(); j++)
 		{
 			delete ms_emuArchives[j];
 		}
@@ -469,9 +584,9 @@ CFArchive::~CFArchive()
 void CFArchive::InitRootArchiveAndEnvironment(const string& pathname, const string& emuImageDir)
 	throw (CFException)
 {
-	TRBEGIN("CFArchive::CFArchive");
+	TRBEGIN("CFArchive::InitRootArchiveAndEnvironment");
 	TRACE(<< "pathname=" << pathname);
-	TRACE(<< "emuImageDir" << emuImageDir);
+	TRACE(<< "emuImageDir=" << emuImageDir);
 
 	m_state = ROOT; //root archive will always have this state
 
@@ -480,7 +595,7 @@ void CFArchive::InitRootArchiveAndEnvironment(const string& pathname, const stri
 	::umask(0022);
 #endif
 
-	EvaluateEnvironmentVariables();
+	ReadSettings();
 
 	//first work in temp dir - will be changed later to dir in tree
 	ASSERT(!ms_tmpdir.empty());
@@ -548,11 +663,11 @@ void CFArchive::InitRootArchiveAndEnvironment(const string& pathname, const stri
 
 void CFArchive::CleanUpRootArchiveFilesystem(void) const throw (CFException)
 {
-	for (int i = 0; i < m_files.size(); i++)
+	ASSERT(m_state == ROOT); //only allowed from root archive
+	for (vector<CFArchive *>::size_type i = 0; i < m_files.size(); i++)
 	{
 		m_files[i]->CleanUpFilesystem();
 	}
-	ASSERT(m_state == ROOT); //only allowed from root archive
 	CleanUpDirectory(ms_miscDir);
 	//for the mother of all archives delete the emu image dir
 	//only if this is wanted (no explicit image dir at command line)
@@ -561,12 +676,21 @@ void CFArchive::CleanUpRootArchiveFilesystem(void) const throw (CFException)
 		CleanUpDirectory(ms_emuImageDir);
 	}
 	CleanUpDirectory(m_workdir); //directory of this archive
+	//now we test if whole temporary directory structure is deleted
+	struct stat buf;
+	if (::stat(m_workdir.c_str(), &buf) != -1)
+	{
+		//top directory still there - give warning
+		exc = CFException(CFException::FILE_REMOVAL_FAILED,
+			__LINE__, __FILE__, m_workdir);		
+		exc.WriteOutExceptionWithWarningHeader(cerr);
+	}
 }
 
 
 void CFArchive::CleanUpFilesystem(void) const throw (CFException)
 {
-	for (int i = 0; i < m_files.size(); i++)
+	for (vector<CFArchive *>::size_type i = 0; i < m_files.size(); i++)
 	{
 		m_files[i]->CleanUpFilesystem();
 	}
@@ -580,6 +704,7 @@ void CFArchive::CleanUpDirectory(string dirName) throw (CFException)
 	TRACE(<< "dirName=" << dirName);
     DIR *pDir;
     struct dirent *pEntry;
+	CFException excWarning;
 
 	//this may happen if exception occurs while constructing a CFArchive
 	//directories are set while constructing
@@ -627,21 +752,27 @@ void CFArchive::CleanUpDirectory(string dirName) throw (CFException)
 			{
 #ifdef _MSC_VER
 				//only writable files can be removed for M$
-				::chmod((dirName + "/" + pEntry->d_name).c_str(), _S_IWRITE);
+				(void)::chmod((dirName + "/" + pEntry->d_name).c_str(), _S_IWRITE);
 #endif
-				::remove((dirName + "/" + pEntry->d_name).c_str());
+				if (::remove((dirName + "/" + pEntry->d_name).c_str()))
+				{
+					excWarning = CFException(CFException::FILE_REMOVAL_FAILED,
+					__LINE__, __FILE__, dirName + "/" + pEntry->d_name, "", errno);
+				}
 			}
 			else
 			{
 				//should be cleaned by previous subsequent destructors
 				if (S_ISDIR(buf.st_mode))
 				{
-					::rmdir((dirName + "/" + pEntry->d_name).c_str());
+					//do not check success of dir removal
+					//removal of top dir is checked later on
+					(void)::rmdir((dirName + "/" + pEntry->d_name).c_str());
 				}
 				else
 				{
 					//unexpected file;
-					::closedir(pDir);
+					(void)::closedir(pDir);
 					exc = CFException(CFException::UNEXPECTED_FILE_TYPE,
 						__LINE__, __FILE__, dirName + "/" + pEntry->d_name
 						, CFException::IntToString(buf.st_mode));
@@ -654,7 +785,7 @@ void CFArchive::CleanUpDirectory(string dirName) throw (CFException)
 	} //END try
 	catch (...)
 	{
-		::closedir(pDir); //close the directory if processing fails
+		(void)::closedir(pDir); //close the directory if processing fails
 		throw;
 	}
 
@@ -664,12 +795,19 @@ void CFArchive::CleanUpDirectory(string dirName) throw (CFException)
 			__LINE__, __FILE__, dirName, "", errno);
 		throw exc;
 	}
-	::rmdir(dirName.c_str());
+	//do not check success of dir removal
+	//removal of top dir is checked later on
+	(void)::rmdir(dirName.c_str());
+	//if we have encountered a warning write it out
+	if (excWarning.GetExceptionId() != CFException::EXCEPTION_INVALID)
+	{
+		excWarning.WriteOutExceptionWithWarningHeader(cerr);
+	}
 	TRreturn;		
 }
 
-
-void CFArchive::SetPathComponents(const string& pathname)
+//SplitPathname may throw exception
+void CFArchive::SetPathComponents(const string& pathname) throw (CFException)
 {
 	TRBEGIN("CFArchive::SetPathComponents");
 	TRACE(<< "pathname=" << pathname);
@@ -682,52 +820,19 @@ void CFArchive::SetPathComponents(const string& pathname)
 
 
 //creator of any archive
-CFArchive *CFArchive::CreateSubClassObject(const string& path, const int blockSize) const
+CFArchive *CFArchive::CreateSubClassObject(const string& path
+	, const unsigned int blockSize) const throw (CFException)
 {
 	TRBEGIN("CFArchive::CreateSubClassObject");
 	TRACE(<< "pathname=" << path << " blockSize=" << blockSize);
 	CFArchive *pSubClass = NULL;
 	
-	if (CFZipC64Archive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFZipC64Archive(path, blockSize);
-	}
-	else if (CFLhaArchive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFLhaArchive(path, blockSize);
-	}
-	else if (CFLnxArchive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFLnxArchive(path, blockSize);
-	}
-	else if (CFD64Archive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFD64Archive(path, blockSize);
-	}
-	else if (CFT64Archive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFT64Archive(path, blockSize);
-	}
-	else if (CFZipArchive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFZipArchive(path, blockSize);
-	}
-	else if (CFGzipArchive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFGzipArchive(path, blockSize);
-	}
-	else if (CFX64Archive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFX64Archive(path, blockSize);
-	}
-	else if (CFPc64Archive::CheckFileIsOfClass(path))
-	{
-		pSubClass = new CFPc64Archive(path, blockSize);
-	}
-
 	//check if file is of generic archive class
 	//iterate over the user defined generic archive classes
-	for (int i = 0; i < CFGenericArchive::ms_genericArchives.size(); i++)
+	//WE DO THIS FIRST TO HAVE A REPLACEMENT FOR EXTERNAL HELPER
+	//MODE. GENERIC ARCHIVE ON BUILTIN IS PRIOR TO BUILTIN.
+	for (vector<CFGenericArchive>::size_type i = 0
+		; i < CFGenericArchive::ms_genericArchives.size(); i++)
 	{
 		if (CFGenericArchive::ms_genericArchives[i]
 			.CheckFileIsOfClass(path))
@@ -746,12 +851,53 @@ CFArchive *CFArchive::CreateSubClassObject(const string& path, const int blockSi
 			break;
 		}
 	}
+	if (pSubClass == NULL)
+	{
+		//only check for builtins after generic archives did not match
+		if (CFZipC64Archive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFZipC64Archive(path, blockSize);
+		}
+		else if (CFLhaArchive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFLhaArchive(path, blockSize);
+		}
+		else if (CFLnxArchive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFLnxArchive(path, blockSize);
+		}
+		else if (CFD64Archive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFD64Archive(path, blockSize);
+		}
+		else if (CFT64Archive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFT64Archive(path, blockSize);
+		}
+		else if (CFZipArchive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFZipArchive(path, blockSize);
+		}
+		else if (CFGzipArchive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFGzipArchive(path, blockSize);
+		}
+		else if (CFX64Archive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFX64Archive(path, blockSize);
+		}
+		else if (CFPc64Archive::CheckFileIsOfClass(path))
+		{
+			pSubClass = new CFPc64Archive(path, blockSize);
+		}
+	}
 
 	//nice to have a final file
 	if (!pSubClass)
 	{
 		pSubClass = new CFFinalArchive(path, blockSize);
 	}
+	ASSERT(pSubClass != NULL);
 	TRACE(<< "Created object is of class: " << typeid(*pSubClass).name());
 	//son has to remember who his parent is (to traverse up)
 	pSubClass->m_pParentArchive = const_cast<CFArchive *>(this);
@@ -768,39 +914,6 @@ CFArchive *CFArchive::CreateSubClassObject(const string& path, const int blockSi
 }
 
 
-void CFArchive::Extract(void) throw (CFException)
-{
-	TRBEGIN("CFArchive::Extract");
-	//if already extracted or mother of all archives do no extraction
-	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
-		TRreturn;
-
-	ASSERT(!m_extractCommand.empty())
-			
-	//extract this archive into a new directory to build hirarchy
-	m_workdir = NameAndCreateTmpDir();
-	
-	//build the extract command
-	//replace placeholders by real pathes
-	//%s1 is working dir and %s2 is pathname of archive
-	//%s3 is the filename (without dir and extension) of the
-	//archive this is used for C1514 extraction to produce
-	//the name of a D64 image
-	m_extractCommand = ReplacePlaceHoldersByStrings(m_extractCommand,
-		m_workdir, m_pathname, m_filename);
-	//execute archive extraction
-	TRACE(<< "Execute command: " << m_extractCommand);
-	if (SystemRedirectOutput(m_extractCommand.c_str()) != 0)
-	{
-		exc = CFException(CFException::SYSTEM_CALL_FAILED,
-			__LINE__, __FILE__, m_extractCommand);
-		throw exc;
-	}
-	TransitionTo(EXTRACTED);
-	TRreturn;
-}
-
-
 void CFArchive::ProcessRootArchive(void) throw (CFException)
 {
 	TRBEGIN("CFArchive::ProcessRootArchive");
@@ -808,6 +921,34 @@ void CFArchive::ProcessRootArchive(void) throw (CFException)
 	ASSERT(m_state == ROOT);
 	ASSERT(!m_pParentArchive);
 
+#ifdef NOCOPY
+
+	for (vector<CFArchive *>::size_type i = 0; i < m_files.size(); i++)
+	{
+		try {
+		m_files[i]->ProcessFilesInArchive();
+		}
+		catch (CFException& actExc)
+		{
+			//on level of root archive we declare input file processing as failed
+			//and continue with next input file (do not throw exception)
+			actExc.WriteOutExceptionWithWarningHeader(cerr);
+		}
+	}
+
+	//flush potentially opened disk image with final archives of this archive
+	CloseActualEmuImage();
+	
+	ProduceWarningsForIncompleteMultipart();
+
+	if (ms_emuArchives.size() == 0)
+	{
+		exc = CFException(CFException::APPLICATION_ERROR,
+			__LINE__, __FILE__, "", "");
+		throw exc;
+	}
+#else
+
 	bool bProcessingFailure = false;
 
 	try {
@@ -826,6 +967,7 @@ void CFArchive::ProcessRootArchive(void) throw (CFException)
 			__LINE__, __FILE__, "", "");
 		throw exc;
 	}
+#endif
 	TRreturn;
 }
 
@@ -837,7 +979,7 @@ bool CFArchive::ProcessSingleFileOfArchive(struct dirent *pEntry)
 	TRACE(<< "Filename: " << pEntry->d_name);
 	struct stat buf;
 
-	if (stat((m_workdir + "/" + pEntry->d_name).c_str(), &buf) != 0)
+	if (::stat((m_workdir + "/" + pEntry->d_name).c_str(), &buf) != 0)
 	{
 		exc = CFException(CFException::FILE_STAT_FAILED,
 			__LINE__, __FILE__, m_workdir + "/" + pEntry->d_name
@@ -848,34 +990,9 @@ bool CFArchive::ProcessSingleFileOfArchive(struct dirent *pEntry)
 	if (S_ISREG(buf.st_mode))
 	{
 		string authenticName = pEntry->d_name;
-		//Normalizing the filename can create a big problem:
-		//as the directory is scanned a filename a move may
-		//lead to a rescan of the moved file which will cause double
-		//processing of this file. so we do it only if -x mode set.
-		if (ms_bUseExternalHelper)
-		{
-			//The following lines are a hack because the tools refuse
-			//working with a few special characters in filename. So we
-			//"normalize" the filename first.
-			string oldname = pEntry->d_name;
-			authenticName = BuildHostConformFilename(oldname);
-			if (oldname != authenticName)
-			{
-				TRACE(<< "Filename convention violated: move "
-					<< oldname << " to " << authenticName);
-				oldname = m_workdir + "/" + oldname;
-				string newname = m_workdir + "/" + authenticName;
-				if (::rename(oldname.c_str(), newname.c_str()))
-				{
-					exc = CFException(CFException::FILE_RENAME_FAILED,
-						__LINE__, __FILE__, oldname, newname, errno);
-					throw exc;
-				}
-			}
-		}
 		
 		//calculate the blocksize of the file
-		int blockSize = (buf.st_size +  CFD64Archive::BLOCKSIZE - 1) / CFD64Archive::BLOCKSIZE;
+		unsigned int blockSize = ((unsigned int)buf.st_size +  BLOCKSIZE_NETTO - 1) / BLOCKSIZE_NETTO;
 		TRACE(<< "Processing file " << (m_workdir + "/" +
 				authenticName));
 		//create the right object and queue it
@@ -897,8 +1014,8 @@ void CFArchive::ProcessFilesInArchive(void) throw (CFException)
 	if (m_state != EXTRACTED)
 		Extract(); //to process files we need extracted archive
 
-    DIR *pDir;
-    struct dirent *pEntry;
+	DIR *pDir;
+	struct dirent *pEntry;
 		
 	if ((pDir = ::opendir(m_workdir.c_str())) == NULL)
 	{
@@ -911,8 +1028,8 @@ void CFArchive::ProcessFilesInArchive(void) throw (CFException)
 	while ((pEntry = ::readdir(pDir)) != NULL)
 	{
 		try {
-		//if dir entry is an archive create it, process it further
-		ProcessSingleFileOfArchive(pEntry);
+		//if dir entry is an archive create archive object and process it further
+		(void)ProcessSingleFileOfArchive(pEntry);
 		} // END try
 		catch (CFException& actExc)
 		{
@@ -925,7 +1042,7 @@ void CFArchive::ProcessFilesInArchive(void) throw (CFException)
 			else
 			{
 				//on any other level we clean up and go up
-				::closedir(pDir); //close the directory if processing fails
+				(void)::closedir(pDir); //close the directory if processing fails
 				CloseActualEmuImage();
 				throw;
 			}
@@ -954,7 +1071,14 @@ void CFArchive::ProcessFilesInArchive(void) throw (CFException)
 }
 
 
-void CFArchive::CloseActualEmuImage(void)
+void CFArchive::Extract(void) throw (CFException)
+{
+	//this must not happen - how should we extract an abstract archive?
+	ASSERT(0);
+}
+
+
+void CFArchive::CloseActualEmuImage(void) throw (CFException)
 {
 	TRBEGIN("CFArchive::CloseActualEmuImage");
 
@@ -1005,6 +1129,7 @@ bool CFArchive::TraverseAllArchivesOfLevelSearchingMultipart(
 				CFMultiPart *pMultiPartCandidate
 					= dynamic_cast<CFMultiPart *>(&candidate);
 				ASSERT(pMultiPartCandidate);
+				ASSERT(*i != NULL);
 				//1. is archive of same type as candidate?
 				//2. is the multipart archive to merge incomplete?
 				if ((typeid(**i) == typeid(candidate))
@@ -1017,7 +1142,7 @@ bool CFArchive::TraverseAllArchivesOfLevelSearchingMultipart(
 					{
 						//now remove the previously merged archive
 						CFArchive *delArchive = *i;
-						m_files.erase(i); //from child array
+						i = m_files.erase(i); //from child array
 						delete delArchive; //and from dynamic memory
 						//i points now one element further than before erasure
 						//do not increment
@@ -1050,7 +1175,7 @@ public:
 	int operator() (CFArchive*& arch)
 	{
 		CFMultiPart const *pMultiPart;
-		if ((pMultiPart = dynamic_cast<const CFMultiPart *>(arch))
+		if (((pMultiPart = dynamic_cast<const CFMultiPart *>(arch)) != NULL)
 			&& !pMultiPart->TestCompleteness())
 			return true;
 		return false;
@@ -1089,6 +1214,12 @@ const string& CFArchive::BuildUniqueEmuImageName(void) const throw (CFException)
 {
 	TRBEGIN("CFArchive::BuildUniqueEmuImageName");
 	static string uniqueFilename;
+	
+	//this vector to keep track of already produced emu image
+	//names. this is necessary because a requested name does
+	//NOT mean that this name already appears in filesystem
+	//so we have to keep track of the names
+	static vector<string> usedEmuImageNames;
 
 	//a new emu image has been requested - increment the counter for emu images
 	//remember the request order of D64 image
@@ -1111,20 +1242,25 @@ const string& CFArchive::BuildUniqueEmuImageName(void) const throw (CFException)
 		ASSERT(!uniqueFilename.empty());
 		ASSERT(ms_emuImageDir.empty() || ms_emuImageDir[ms_emuImageDir.size() - 1] == '/');
 		//make filename lowercase
-		for (int i = 0; i < uniqueFilename.size(); i++)
-			uniqueFilename[i] = tolower(uniqueFilename[i]);
+		for (string::size_type i = 0; i < uniqueFilename.size(); i++)
+			uniqueFilename[i] = (char)tolower(uniqueFilename[i]);
 		//make filename command line conform (e.g. remove blanks)
 		uniqueFilename = BuildHostConformFilename(uniqueFilename);
 		//build absolute path
 		uniqueFilename = ms_emuImageDir + uniqueFilename + ".d64";
 		//throws exception if fails
-		uniqueFilename = ::UniqueFileNameGenerator(uniqueFilename);
+		//check within filesystem AND the vector of already
+		//used names.
+		uniqueFilename = ::UniqueFileNameGenerator(uniqueFilename
+			, &usedEmuImageNames);
+		//we store the unique filename because it is NOT yet
+		//present in filesystem but ALREADY occupied!
+		usedEmuImageNames.push_back(uniqueFilename);
 	}
 	else
 	{
 		//just count the emu images from target000 - target<n>
 		strstream imageName;
-		//increment the emu archive count
 		imageName << ms_emuImageDir << ms_emuArchiveBasename << setw(3) << setfill('0')
 			<< ms_emuArchiveCount << ".d64" << '\0';
 		uniqueFilename = imageName.str();
@@ -1142,6 +1278,7 @@ string& CFArchive::NameAndCreateTmpDir(void) throw (CFException)
 	resultString = m_workdir + "/" + m_filename + "XXXXXX";
 	char *tmp = new char[resultString.size() + 1]; //one for \0
 
+	ASSERT(tmp != NULL);
 	::strcpy(tmp, resultString.c_str());
 	if (::mktemp(tmp) == NULL) //make unique directory name
 	{
@@ -1231,7 +1368,7 @@ void CFArchive::CopyThisArchiveToDirectory(const string& directory) throw (CFExc
 
 	while (in.get(ch))
 	{
-		out.put(ch);
+		(void)out.put(ch);
 	}
 
 	if (in.bad())
@@ -1265,8 +1402,8 @@ const CFArchive& CFArchive::GetFirstEmuImageArchive(void)
 	//emu image for a.prg is still pending and queued AT LAST!
 	//so I have introduced an counter for image requests which
 	//actually stores the request order which matters!
-	int nFirstArchiveIndex = 0;
-	for (int i = 1; i < ms_emuArchives.size(); i++)
+	vector<CFArchive *>::size_type  nFirstArchiveIndex = 0;
+	for (vector<CFArchive *>::size_type i = 1; i < ms_emuArchives.size(); i++)
 	{
 		if (ms_emuArchives[i]->m_nRequestOrderIndex
 			< ms_emuArchives[nFirstArchiveIndex]->m_nRequestOrderIndex)
@@ -1297,7 +1434,7 @@ string CFArchive::ConvertFilenameUnixToWin(const string& filename)
 {
 	string retString = filename;
 	
-	for (int i = 0; i < retString.size(); i++)
+	for (string::size_type i = 0; i < retString.size(); i++)
 	{
 		if (retString[i] == '/')
 		{
@@ -1319,7 +1456,7 @@ string CFArchive::ConvertFilenameUnixToWinShort(const string& filename)
 	if (shortName == NULL)
 	{
 		exc = CFException(CFException::MEMORY_ALLOCATION_FAILED,
-			__LINE__, __FILE__, CFException::IntToString(retString.size() + 1), "");
+			__LINE__, __FILE__, CFException::IntToString((int)retString.size() + 1), "");
 		throw exc;
 	}
 	if (GetShortPathName(retString.c_str(), shortName, retString.size() + 1) == 0)
@@ -1348,26 +1485,10 @@ int CFArchive::BuildBatchFileAndExecuteSystem(const string& command)
 	ASSERT(CFArchive::ms_miscDir.empty()
 		|| CFArchive::ms_miscDir[CFArchive::ms_miscDir.size() - 1] == '/');
 
-	//if no ; is found execute single command without batchfile
-	if (command.find(';') == string::npos)
-	{
-		return system(command.c_str());
-	}
+	string batchFileName = UniqueFileNameGenerator(CFArchive::ms_miscDir + "batchfile.bat");
 
-	string batchFileName = CFArchive::ms_miscDir + "baXXXXXX";
-	char *tmpFilename = new char[batchFileName.size() + 1];
-	::strcpy(tmpFilename, batchFileName.c_str());
-	if (::mktemp(tmpFilename) == NULL)
-	{
-		delete [] tmpFilename;
-		exc = CFException(CFException::UNIQUE_NAME_CREATION_FAILED,
-			__LINE__, __FILE__, batchFileName, "", errno);
-		throw exc;
-	}	
-
-	batchFileName = (string)tmpFilename + ".bat";
-	delete [] tmpFilename;
-		
+	//strange enough newlines were not translated into \r\n
+	//when opening in textmode, so I am using binary
 	ofstream batch(batchFileName.c_str(), ios::out | ios::binary);
 	if (!batch)
 	{
@@ -1382,25 +1503,8 @@ int CFArchive::BuildBatchFileAndExecuteSystem(const string& command)
 	{
 		string singleCommand;
 		//get a command seperated by ; or eof
-		getline(commandStream, singleCommand, ';');
-		//ATTENTION: % character in M$ batchfiles must be % quoted (%%)
-		int pos = 0;
-		while ((pos = singleCommand.find("%", pos)) != string::npos)
-		{
-			//from the left to the percent (included)
-			string leftEqualPercent = singleCommand.substr(0, pos + 1);
-			//from the percent (included) to the right
-			string rightEqualPercent = singleCommand.substr(pos
-				, singleCommand.size() - pos);
-			//this is the percent doubling
-			singleCommand = leftEqualPercent + rightEqualPercent;
-			//make sure that originally found percent and appended
-			//percent are not found
-			pos += 2;
-			if (pos >= singleCommand.size())
-				pos = string::npos;
-		}
-		batch << singleCommand <<"\r\n";
+		(void)getline(commandStream, singleCommand, ';');
+		batch << singleCommand << "\r\n";
 	}
 	if (!commandStream.eof() || !batch)
 	{
@@ -1414,8 +1518,8 @@ int CFArchive::BuildBatchFileAndExecuteSystem(const string& command)
 	//M$ will return 0 if command is not found - no workaround for this error
 	int ret = system(CFArchive::ConvertFilenameUnixToWin(batchFileName).c_str());
 
-	//now remove the batchfile
-	if (::remove(batchFileName.c_str()))
+	//now remove the batchfile (only if indicated)
+	if (ms_bCleanup && ::remove(batchFileName.c_str()))
 	{
 		exc = CFException(CFException::FILE_REMOVAL_FAILED,
 			__LINE__, __FILE__, batchFileName, "", errno);
@@ -1459,6 +1563,7 @@ int CFArchive::SystemRedirectOutput(const char *command)
 		filename = ms_miscDir + "redirectedXXXXXX";
 
 		char *tmpFilename = new char[filename.size() + 1];
+		ASSERT(tmpFilename != NULL);
 		::strcpy(tmpFilename, filename.c_str());
 		if (::mktemp(tmpFilename) == NULL)
 		{
@@ -1487,7 +1592,7 @@ int CFArchive::SystemRedirectOutput(const char *command)
 	int oldStderr = ::dup(handleStderr);
 	if (oldStderr < 0)
 	{
-		::close(oldStdout);
+		(void)::close(oldStdout);
 		exc = CFException(CFException::FILE_DUP_FAILED,
 			__LINE__, __FILE__, CFException::IntToString(handleStderr)
 			, "", errno);
@@ -1495,10 +1600,10 @@ int CFArchive::SystemRedirectOutput(const char *command)
 	}
 		
 	FILE *newOut = ::fopen(filename.c_str(), "ab");
-	if (!newOut)
+	if (newOut == NULL)
 	{
-		::close(oldStdout);
-		::close(oldStderr);
+		(void)::close(oldStdout);
+		(void)::close(oldStderr);
 		exc = CFException(CFException::FILE_OPEN_FAILED,
 			__LINE__, __FILE__, filename, "append writing", errno);
 		throw exc;
@@ -1506,9 +1611,9 @@ int CFArchive::SystemRedirectOutput(const char *command)
 	      
 	if (dup2(fileno(newOut), handleStdout) < 0)
 	{
-		::fclose(newOut);
-		::close(oldStdout);
-		::close(oldStderr);
+		(void)::fclose(newOut);
+		(void)::close(oldStdout);
+		(void)::close(oldStderr);
 		exc = CFException(CFException::FILE_DUP2_FAILED,
 			__LINE__, __FILE__, CFException::IntToString(fileno(newOut))
 			, CFException::IntToString(handleStdout), errno);
@@ -1516,10 +1621,10 @@ int CFArchive::SystemRedirectOutput(const char *command)
 	}
 	if (::dup2(fileno(newOut), handleStderr) < 0)
 	{
-		::fclose(newOut);
-		::dup2(oldStdout, handleStdout);
-		::close(oldStdout);
-		::close(oldStderr);
+		(void)::fclose(newOut);
+		(void)::dup2(oldStdout, handleStdout);
+		(void)::close(oldStdout);
+		(void)::close(oldStderr);
 		exc = CFException(CFException::FILE_DUP2_FAILED,
 			__LINE__, __FILE__, CFException::IntToString(fileno(newOut))
 			, CFException::IntToString(handleStderr), errno);
@@ -1538,21 +1643,21 @@ int CFArchive::SystemRedirectOutput(const char *command)
 	catch (...)
 	{
 		//make sure that in case of exception redirection is taken back
-		::fclose(newOut);
-		::dup2(oldStdout, handleStdout);
-		::dup2(oldStderr, handleStderr);
-		::close(oldStdout);
-		::close(oldStderr);
+		(void)::fclose(newOut);
+		(void)::dup2(oldStdout, handleStdout);
+		(void)::dup2(oldStderr, handleStderr);
+		(void)::close(oldStdout);
+		(void)::close(oldStderr);
 		throw;
 	}
 
 	//we asume that the following calls go well and so errno is not
 	//affected and reflects system()
-	::fclose(newOut);
-	::dup2(oldStdout, handleStdout);
-	::dup2(oldStderr, handleStderr);
-	::close(oldStdout);
-	::close(oldStderr);
+	(void)::fclose(newOut);
+	(void)::dup2(oldStdout, handleStdout);
+	(void)::dup2(oldStderr, handleStderr);
+	(void)::close(oldStdout);
+	(void)::close(oldStderr);
 	
 	return ret;
 }
@@ -1573,32 +1678,40 @@ static DIR *opendir(const char *name)
 {
 	//allocate a new directory handle
 	DIR *newDIR = new DIR;
+	ASSERT(newDIR != NULL);
 
 	char pwd[MAX_PATH];
 	if (!GetCurrentDirectory(MAX_PATH, pwd))
 	{
-		errno = GetLastError();
+		errno = (int)GetLastError();
 		delete newDIR;
 		return NULL;
 	}
 
+	//set the directory to scan
 	if (!SetCurrentDirectory(name))
 	{
-		errno = GetLastError();
+		errno = (int)GetLastError();
 		delete newDIR;
 		return NULL;
 	}
 
 	*newDIR = FindFirstFile("*.*", &dirFileData);
-
-	SetCurrentDirectory(pwd);
-
 	if (*newDIR == INVALID_HANDLE_VALUE)
 	{
-		errno = GetLastError(); //set system error as opendir would do for UNIX
+		errno = (int)GetLastError(); //set system error as opendir would do for UNIX
 		delete newDIR;
 		return NULL;
 	}
+
+	//set back the current directory
+	if (!SetCurrentDirectory(pwd))
+	{
+		errno = (int)GetLastError(); //set system error as opendir would do for UNIX
+		delete newDIR;
+		return NULL;
+	}
+
 	return newDIR;
 }
 
@@ -1615,7 +1728,7 @@ static int closedir(DIR *dir)
 		return 0;
 	}
 	*dir = NULL;
-	errno = GetLastError(); //set system error as closedir would do for UNIX
+	errno = (int)GetLastError(); //set system error as closedir would do for UNIX
 	return 1;
 }
 
@@ -1639,7 +1752,7 @@ static struct dirent *readdir(DIR *dir)
 		if (GetLastError() != ERROR_NO_MORE_FILES)
 		{
 			//closedir will give an error for this one
-			FindClose(*dir);
+			(void)FindClose(*dir);
 			*dir = NULL;
 		}
 		return NULL;
