@@ -33,24 +33,23 @@
 #endif
 
 #include <stdio.h>
+#include <errno.h>
 #ifdef _MSC_VER
 #include <io.h>
-#include <sys/stat.h>
 #endif
+#include <sys/stat.h>
 
 #include <string>
 #include <vector>
-#ifdef _MSC_VER
-#include <typeinfo.h>
 #include <iostream>
 #include <strstream>
 #include <fstream>
+#ifdef _MSC_VER
+#include <typeinfo.h>
 #else
 #include <typeinfo>
-#include <iostream.h>
-#include <strstream.h>
-#include <fstream.h>
 #endif
+
 using namespace std;
 
 #include "Exception.h"
@@ -66,8 +65,7 @@ using namespace std;
 #include "vdrive.h" //VICE interface
 
 
-int CFD64Archive::BLOCKSIZE = 254; //size of data without linkage header
-int CFD64Archive::BLOCKCAPACITY = 664;
+unsigned int CFD64Archive::BLOCKCAPACITY = 664;
 
 
 //-------------------------------------------------------------------
@@ -75,7 +73,7 @@ int CFD64Archive::BLOCKCAPACITY = 664;
 //-------------------------------------------------------------------
 
 
-bool CFD64Archive::CheckFileIsOfClass(const string& filename)
+bool CFD64Archive::CheckFileIsOfClass(const string& filename) throw (CFException)
 {
 	string extension, invalid;
 
@@ -91,21 +89,54 @@ bool CFD64Archive::CheckFileIsOfClass(const string& filename)
 void CFD64Archive::Extract(void) throw (CFException)
 {
 	TRBEGIN("CFD64Archive::Extract");
+
+	const unsigned int D64_SIZE = 174848; //35 track D64
+	const unsigned int D64_ERR_SIZE = 175531; //35 track D64 + error info
+	const unsigned int D64EX_SIZE = 196608; //40 track D64
+	const unsigned int D64EX_ERR_SIZE = 197376; //40 track D64 + error info
+
 	//if already extracted or mother of all archives do no extraction
 	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
 		TRreturn;
 
-	ASSERT(m_extractCommand.empty())
+	struct stat buf;
+
+	if (stat(m_pathname.c_str(), &buf) != 0)
+	{
+		exc = CFException(CFException::FILE_STAT_FAILED,
+			__LINE__, __FILE__, m_pathname, "", errno);
+		throw exc;
+	}
 			
 	//extract this archive into a new directory to build hirarchy
 	m_workdir = NameAndCreateTmpDir();
 	
 	//extract by builtin diskimage handler
-	CFDiskImage extractDisk;
-	extractDisk.Attach(m_pathname);
-	m_bMoreFilesThanExtractedOnDisk = extractDisk.ExtractAllFiles(m_workdir);
-
-	m_extractedFiles = extractDisk.GetVectorOfExtractedFiles();
+	switch (buf.st_size)
+	{
+	case D64_SIZE:
+	case D64_ERR_SIZE:
+		{
+		CFDiskImageD64 extractDisk;
+		extractDisk.Attach(m_pathname);
+		m_bMoreFilesThanExtractedOnDisk = extractDisk.ExtractAllFiles(m_workdir);
+		m_extractedFiles = extractDisk.GetVectorOfExtractedFiles();
+		}	
+		break;
+	case D64EX_SIZE:
+	case D64EX_ERR_SIZE:
+		{
+		CFDiskImageD64Ex extractDisk;
+		extractDisk.Attach(m_pathname);
+		m_bMoreFilesThanExtractedOnDisk = extractDisk.ExtractAllFiles(m_workdir);
+		m_extractedFiles = extractDisk.GetVectorOfExtractedFiles();
+		}
+		break;
+	default:
+		exc = CFException(CFException::DISK_IMAGE_SIZE_INCORRECT,
+			__LINE__, __FILE__, m_pathname, "size unexpected - not in (174848, 175531, 196608, 197376) bytes");
+		throw exc;
+	}
 
 	TransitionTo(EXTRACTED);
 	TRreturn;
@@ -129,12 +160,14 @@ void CFD64Archive::ProcessFilesInArchive(void) throw (CFException)
 		d64Name = BuildUniqueEmuImageName();
 	}
 
+	bool bAtLeastOneResolvableArchie = false;
+
 	//extraction did store all extracted files - iterate over vector
-	for (int i = 0; i < m_extractedFiles.size(); i++)
+	for (vector<CFExtractedFileInfo>::size_type i = 0; i < m_extractedFiles.size(); i++)
 	{
 		TRACE(<< "Processing file " << m_extractedFiles[i].GetFilename());
-		int actualBlockSize = (m_extractedFiles[i].GetFileSize() + CFD64Archive::BLOCKSIZE - 1)
-			/ CFD64Archive::BLOCKSIZE;
+		unsigned int actualBlockSize = (m_extractedFiles[i].GetFileSize() + BLOCKSIZE_NETTO - 1)
+			/ BLOCKSIZE_NETTO;
 		//create the right object and queue it
 		CFArchive *newEntry = CreateSubClassObject(m_extractedFiles[i].GetFilename()
 			, actualBlockSize);
@@ -152,22 +185,67 @@ void CFD64Archive::ProcessFilesInArchive(void) throw (CFException)
 		else
 		{
 			//descent into archive
+			bAtLeastOneResolvableArchie = true;
 			newEntry->ProcessFilesInArchive();
 		}
 	}
 
-	//rename the D64 image for the emulator if indicated
+	//if we had no resolvable archive on disk and the disk appears to be empty
+	//still pass it to the emulator:
+	//there are lot of games that do direct block access on disk without an
+	//1541 compatible filesystem.
+	if (d64Name.empty() && !bAtLeastOneResolvableArchie)
+		d64Name = BuildUniqueEmuImageName();
+
+	//rename the D64 image for the emulator use if indicated
 	if (!d64Name.empty())
 	{
-		//rename the D64 file for emulator use
 		if (d64Name != m_pathname) //if file is named right do not rename
 		{
+#ifdef NOCOPY
+			//copy source archive to temporary archive
+			fstream in(m_pathname.c_str(), ios::in | ios::binary);
+			if (!in)
+			{
+				exc = CFException(CFException::FILE_OPEN_FAILED,
+					__LINE__, __FILE__, m_pathname, "reading");
+				throw exc;
+			}
+	
+			fstream out(d64Name.c_str(), ios::out | ios::binary);
+			if (!out)
+			{
+				exc = CFException(CFException::FILE_OPEN_FAILED,
+					__LINE__, __FILE__, d64Name, "write from scratch");
+				throw exc;
+			}	
+			char ch;
+
+			while (in.get(ch))
+			{
+				(void)out.put(ch);
+			}
+
+			if (in.bad())
+			{
+				exc = CFException(CFException::FILE_OPERATION_FAILED,
+					__LINE__, __FILE__, m_pathname, "reading");
+				throw exc;
+			}
+			if (!out)
+			{
+				exc = CFException(CFException::FILE_OPERATION_FAILED,
+					__LINE__, __FILE__, d64Name, "writing");
+				throw exc;
+			}
+#else
+
 #ifdef _MSC_VER
 			//only writable files can be removed for M$
-			::chmod(d64Name.c_str(), _S_IWRITE);
+			(void)::chmod(d64Name.c_str(), _S_IWRITE);
 			//rename does not overwrite already existing files for M$
 			//this may happen for non empty dir specified by -c
-			::remove(d64Name.c_str());
+			(void)::remove(d64Name.c_str());
 #endif
 			if (::rename(m_pathname.c_str(), d64Name.c_str()))
 			{
@@ -175,6 +253,7 @@ void CFD64Archive::ProcessFilesInArchive(void) throw (CFException)
 					__LINE__, __FILE__, m_pathname, d64Name, errno);
 				throw exc;
 			}
+#endif
 		}
 		//we have a new D64 for the emulator
 		CFArchive *tmpArchive = CreateSubClassObject(d64Name);
@@ -201,7 +280,7 @@ void CFD64Archive::ProcessFilesInArchive(void) throw (CFException)
 //-------------------------------------------------------------------
 
 
-bool CFX64Archive::CheckFileIsOfClass(const string& filename)
+bool CFX64Archive::CheckFileIsOfClass(const string& filename) throw (CFException)
 {
 	string extension, invalid;
 
@@ -231,14 +310,14 @@ void CFX64Archive::Extract(void) throw (CFException)
 	string d64Name = m_workdir + "/" + m_filename + ".d64";
 	TRACE(<< "Extract into d64Image=" << d64Name);
 
-    fstream in(m_pathname.c_str(), ios::in | ios::binary);
+	fstream in(m_pathname.c_str(), ios::in | ios::binary);
 	if (!in)
 	{
 		exc = CFException(CFException::FILE_OPEN_FAILED,
 			__LINE__, __FILE__, m_pathname, "reading");
 		throw exc;
 	}
-    fstream out(d64Name.c_str(), ios::out| ios::binary);
+	fstream out(d64Name.c_str(), ios::out| ios::binary);
 	if (!out)
 	{
 		exc = CFException(CFException::FILE_OPEN_FAILED,
@@ -250,11 +329,11 @@ void CFX64Archive::Extract(void) throw (CFException)
 	//Skip 64 bytes of a x64 archive
 	for (int i = 0; i < 64; i++)
 	{
-		in.get(ch);
+		(void)in.get(ch);
 	}
 	while (in.get(ch))
 	{
-		out.put(ch);
+		(void)out.put(ch);
 	}
 	if (in.bad())
 	{
@@ -303,8 +382,6 @@ void CFT64Archive::Extract(void) throw (CFException)
 	//if already extracted or mother of all archives do no extraction
 	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
 		TRreturn;
-
-	ASSERT(m_extractCommand.empty())
 			
 	//extract this archive into a new directory to build hirarchy
 	m_workdir = NameAndCreateTmpDir();
@@ -331,11 +408,12 @@ void CFT64Archive::ProcessFilesInArchive(void) throw (CFException)
 
 	try {
 	//extraction did store all extracted files - iterate over vector
-	for (int i = 0; i < m_extractedFiles.size(); i++)
+	for (vector<CFExtractedFileInfo>::size_type i = 0; i < m_extractedFiles.size(); i++)
 	{
 		TRACE(<< "Processing file " << m_extractedFiles[i].GetFilename());
 		//calculate the blocksize of the file
-		int actualBlockSize = (m_extractedFiles[i].GetFileSize() + CFD64Archive::BLOCKSIZE - 1) / CFD64Archive::BLOCKSIZE;
+		unsigned int actualBlockSize = (m_extractedFiles[i].GetFileSize() + BLOCKSIZE_NETTO - 1)
+			/ BLOCKSIZE_NETTO;
 		//create the right object and queue it
 		CFArchive *newEntry = CreateSubClassObject(m_extractedFiles[i].GetFilename(), actualBlockSize);
 		m_files.push_back(newEntry);
@@ -374,7 +452,7 @@ void CFT64Archive::ProcessFilesInArchive(void) throw (CFException)
 }
 
 
-bool CFT64Archive::CheckFileIsOfClass(const string& filename)
+bool CFT64Archive::CheckFileIsOfClass(const string& filename) throw (CFException)
 {
 	string extension, invalid;
 
@@ -391,18 +469,9 @@ bool CFT64Archive::CheckFileIsOfClass(const string& filename)
 //CFLnxArchive
 //-------------------------------------------------------------------
 
-
-#ifdef _MSC_VER
-string CFLnxArchive::ms_staticExtractCommand
-	= "%d1; cd %l1; starlynx -x %s2 /y";
-#else
-string CFLnxArchive::ms_staticExtractCommand
-	= "cd '%s1' && c1541 -format 'target,64' d64 '%s3.d64' -unlynx '%s2'";
-#endif
-
 static const string LNXPATTERN("USE LYNX TO DISSOLVE THIS FILE");
 
-bool CFLnxArchive::CheckFileIsOfClass(const string& filename)
+bool CFLnxArchive::CheckFileIsOfClass(const string& filename) throw (CFException)
 {
 	string extension, invalid;
 
@@ -419,12 +488,14 @@ bool CFLnxArchive::CheckFileIsOfClass(const string& filename)
 	}
 	
 	char *puffer = new char[LNXPATTERN.size() + 1];
+	ASSERT(puffer != NULL);
 	memset(puffer, 0, LNXPATTERN.size() + 1);
 
-	in.seekg(0x38); //overread until pattern
-	in.read(puffer, LNXPATTERN.size());
+	(void)in.seekg(0x38); //overread until pattern
+	(void)in.read(puffer, (streamsize)LNXPATTERN.size());
 	if (!in)
 	{
+		delete [] puffer;
 		return false;
 	}
 	string pattern(puffer);
@@ -449,25 +520,17 @@ void CFLnxArchive::Extract(void) throw (CFException)
 	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
 		TRreturn;
 
-	if (ms_bUseExternalHelper)
-	{
-		//if old style with external helper base class will take care
-		CFC64Archive::Extract();
-	}
-	else
-	{
-		//extract this archive into a new directory to build hirarchy
-		m_workdir = NameAndCreateTmpDir();
-		
-		//extract by lynx image handler (which uses VICE)
-		CFLynxImage extractLynx;
-		extractLynx.Attach(m_pathname);
-		extractLynx.ExtractAllFiles(m_workdir);
+	//extract this archive into a new directory to build hirarchy
+	m_workdir = NameAndCreateTmpDir();
+	
+	//extract by lynx image handler (which uses VICE)
+	CFLynxImage extractLynx;
+	extractLynx.Attach(m_pathname);
+	extractLynx.ExtractAllFiles(m_workdir);
 
-		m_extractedFiles = extractLynx.GetVectorOfExtractedFiles();
+	m_extractedFiles = extractLynx.GetVectorOfExtractedFiles();
 
-		TransitionTo(EXTRACTED);
-	}
+	TransitionTo(EXTRACTED);
 	TRreturn;
 }
 
@@ -476,59 +539,49 @@ void CFLnxArchive::ProcessFilesInArchive(void) throw (CFException)
 {
 	TRBEGIN("CFLnxArchive::ProcessFilesInArchive");
 
-	if (ms_bUseExternalHelper)
-	{
-		//Extraction will produce a D64 file. This is found in
-		//CFArchive::ProcessFilesInArchive and further processed 
-		CFArchive::ProcessFilesInArchive();
-		//Due to limitations of C1541 unlynx the file directly into a D64
-		//If it wont fit - unlucky you.
-	}
-	else
-	{
-		if (m_state != EXTRACTED)
-			Extract(); //to process files we need extracted archive
+	if (m_state != EXTRACTED)
+		Extract(); //to process files we need extracted archive
 
-		try {
-		//extraction did store all extracted files - iterate over vector
-		for (int i = 0; i < m_extractedFiles.size(); i++)
+	try {
+	//extraction did store all extracted files - iterate over vector
+	for (vector<CFExtractedFileInfo>::size_type i = 0; i < m_extractedFiles.size(); i++)
+	{
+		TRACE(<< "Processing file " << m_extractedFiles[i].GetFilename());
+		//calculate the blocksize of the file
+		unsigned int actualBlockSize = (m_extractedFiles[i].GetFileSize() + BLOCKSIZE_NETTO - 1)
+			/ BLOCKSIZE_NETTO;
+		//create the right object and queue it
+		CFArchive *newEntry = CreateSubClassObject(m_extractedFiles[i].GetFilename(), actualBlockSize);
+		m_files.push_back(newEntry);
+		
+		if (newEntry->Finalized())
 		{
-			TRACE(<< "Processing file " << m_extractedFiles[i].GetFilename());
-			//calculate the blocksize of the file
-			int actualBlockSize = (m_extractedFiles[i].GetFileSize() + CFD64Archive::BLOCKSIZE - 1) / CFD64Archive::BLOCKSIZE;
-			//create the right object and queue it
-			CFArchive *newEntry = CreateSubClassObject(m_extractedFiles[i].GetFilename(), actualBlockSize);
-			m_files.push_back(newEntry);
-			
-			if (newEntry->Finalized())
-			{
-				//store what you know about the CBM file which stands
-				//behind the host file
-				newEntry->SetFileInfoCBM(m_extractedFiles[i]);
-			}
-			//descent into archive
-			newEntry->ProcessFilesInArchive();
-		}
-		} // END try
-		catch (CFException&)
-		{
-			//flush potentially opened disk image with final archives of this archive
-			CloseActualEmuImage();
-			throw;
+			//store what you know about the CBM file which stands
+			//behind the host file
+			newEntry->SetFileInfoCBM(m_extractedFiles[i]);
 		}
+		//descent into archive
+		newEntry->ProcessFilesInArchive();
+	}
+	} // END try
+	catch (CFException&)
+	{
 		//flush potentially opened disk image with final archives of this archive
 		CloseActualEmuImage();
+		throw;
+	}
+	//flush potentially opened disk image with final archives of this archive
+	CloseActualEmuImage();
 
-		TransitionTo(PROCESSED);
+	TransitionTo(PROCESSED);
 
-		if (m_files.size() == 0)
-		{
-			//we have not recognized a single file in archive
-			//this will lead to a warning
-			exc = CFException(CFException::ARCHIVE_EMPTY,
-				__LINE__, __FILE__, GetPathname(), "");
-			exc.WriteOutExceptionWithWarningHeader(cerr);
-		}
+	if (m_files.size() == 0)
+	{
+		//we have not recognized a single file in archive
+		//this will lead to a warning
+		exc = CFException(CFException::ARCHIVE_EMPTY,
+			__LINE__, __FILE__, GetPathname(), "");
+		exc.WriteOutExceptionWithWarningHeader(cerr);
 	}
 	TRreturn;
 }
@@ -539,18 +592,8 @@ void CFLnxArchive::ProcessFilesInArchive(void) throw (CFException)
 //-------------------------------------------------------------------
 
 
-#ifdef _MSC_VER
-string CFZipC64Archive::ms_staticExtractCommand
-	= "%d1; cd %l1; starzip -x %s2 /y";
-#else
-	//we have to strip off the first two characters of the filename to
-	//get a valid d64 filename 1!x.d64 -> x.d64 (else regognized as c64zip)
-string CFZipC64Archive::ms_staticExtractCommand
-	= "cd '%s1' && c1541 -zcreate `expr '%s3.x64' : '..\\(.*\\)'` '%s2'";
-#endif
-
-
 bool CFZipC64Archive::CheckFileIsOfClass(const string& filename)
+	throw (CFException)
 {
 	string zipCode, invalid;
 
@@ -568,17 +611,18 @@ bool CFZipC64Archive::CheckFileIsOfClass(const string& filename)
 }
 
 
-CFZipC64Archive::CFZipC64Archive(const string& path, const int blockSize)
-	: CFC64Archive(path, blockSize, ms_staticExtractCommand)
-	, CFMultiPart(4)
+CFZipC64Archive::CFZipC64Archive(const string& path, const unsigned int blockSize)
+	throw (CFException)
+: CFC64Archive(path, blockSize)
+, CFMultiPart(4)
 {
 	TRBEGIN("CFZipC64Archive::CFZipC64Archive");
 
 	//set the part used for construction to "available"
 	string prefix = m_filename.substr(0, 1);
 	ASSERT((prefix > "0") && (prefix < "5"));
-	int nIndex = prefix[0] - '1';
-	SetPartAvailable(nIndex);
+	unsigned int nIndex = (unsigned char)(prefix[0] - '1');
+	(void)SetPartAvailable(nIndex);
 	//set the pathname of the part used for construction
 	SetPartPathname(nIndex, m_pathname);
 	TRreturn;
@@ -589,120 +633,44 @@ void CFZipC64Archive::Extract(void) throw (CFException)
 {
 	TRBEGIN("CFZipC64Archive::Extract");
 
-	if (ms_bUseExternalHelper)
-	{
-		//old style with external helper
-		ExtractWithExternalHelper();
-	}
-	else
-	{
-		//if already extracted or mother of all archives do no extraction
-		if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
-			TRreturn;
-				
-		//extract this archive into a new directory
-		m_workdir = NameAndCreateTmpDir();
-		
-		string filename = GetFilename();
-		//strip away the [1-4]!
-		filename = filename.substr(2, filename.length() - 2);
-		string outNameD64 = m_workdir + "/" + filename + ".d64";
-
-		ASSERT(GetPartNum() == 4);
-		const char *partNames[4];
-		for (int i = 0; i < GetPartNum(); i++)
-		{
-			partNames[i] = GetPartPathname(i).c_str();
-		}
-		FILE *outFileDescr;
-		//open the output D64 image in C style because access
-		//to foreign sources
-		if ((outFileDescr = fopen(outNameD64.c_str(), "wb")) == NULL)
-		{
-			exc = CFException(CFException::FILE_OPEN_FAILED,
-				__LINE__, __FILE__, outNameD64, "writing", errno);
-			throw exc;
-		}
-		if (zcreate_cmd(outFileDescr, partNames) != FD_OK)
-		{
-			fclose(outFileDescr);
-			::remove(outNameD64.c_str());
-			exc = CFException(CFException::C64ZIP_EXTRACT_FAILED,
-				__LINE__, __FILE__, GetPathname(), "");
-			throw exc;
-		}
-		fclose(outFileDescr);
-
-		TransitionTo(EXTRACTED);	
-	}
-	TRreturn;
-}
-
-
-void CFZipC64Archive::ExtractWithExternalHelper(void) throw (CFException)
-{
-	TRBEGIN("CFZipC64Archive::ExtractWithExternalHelper");
-
 	//if already extracted or mother of all archives do no extraction
 	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
 		TRreturn;
+			
+	//extract this archive into a new directory
+	m_workdir = NameAndCreateTmpDir();
+	
+	string filename = GetFilename();
+	//strip away the [1-4]!
+	filename = filename.substr(2, filename.length() - 2);
+	string outNameD64 = m_workdir + "/" + filename + ".d64";
 
-	//could be we have single parts of multipart from
-	//different directories. If so move them to this directory
-	//because external helper need all parts in one directory
-	int i;
-	//the destination file lies within this directory
-	string destFilename = GetFilename();
-	ASSERT((destFilename.size() > 1) && (destFilename[1] = '!'));
-	string sourceFilename = destFilename; //for moving back
-	for (i = 0; i < 4; i++)
+	ASSERT(GetPartNum() == 4);
+	const char *partNames[4];
+	for (unsigned int i = 0; i < GetPartNum(); i++)
 	{
-		destFilename[0] = '1' + i;
-		string destPathname = GetDirname() + destFilename
-			+ GetExtension();
-		//check if part has to be moved
-		if (destPathname != GetPartPathname(i))
-		{
-			TRACE(<< "Renaming Archive");
-			TRACE(<< "Source: " << GetPartPathname(i));
-			TRACE(<< "Dest. : " << destPathname);
-			if (::rename(GetPartPathname(i).c_str(), destPathname.c_str()))
-			{
-				exc = CFException(CFException::FILE_RENAME_FAILED,
-					__LINE__, __FILE__, GetPartPathname(i), destPathname, errno);
-				throw exc;
-			}
-		}
+		partNames[i] = GetPartPathname(i).c_str();
 	}
-
-	//sitches state to EXTRACTED
-	CFC64Archive::Extract(); //now do the extraction
-
-	//now move back the archives where they came from
-	//this is necessary not to disturb the directory scan which is
-	//going on right now. to find the archive file once again in
-	//this directory would be fatal.
-	for (i = 0; i < 4; i++)
+	FILE *outFileDescr;
+	//open the output D64 image in C style because access
+	//to foreign sources
+	if ((outFileDescr = fopen(outNameD64.c_str(), "wb")) == NULL)
 	{
-		//the source file lies within this directory (has potentially been moved here)
-		sourceFilename[0] = '1' + i;
-		string sourcePathname = GetDirname() + sourceFilename
-			+ GetExtension();
-		//check if part has to be moved BACK where it came from
-		if (sourcePathname != GetPartPathname(i))
-		{
-			TRACE(<< "Renaming Archive");
-			TRACE(<< "Source: " << sourcePathname);
-			TRACE(<< "Dest. : " << GetPartPathname(i));
-			if (::rename(sourcePathname.c_str(), GetPartPathname(i).c_str()))
-			{
-				exc = CFException(CFException::FILE_RENAME_FAILED,
-					__LINE__, __FILE__, sourcePathname, GetPartPathname(i), errno);
-				throw exc;
-			}
-		}
+		exc = CFException(CFException::FILE_OPEN_FAILED,
+			__LINE__, __FILE__, outNameD64, "writing", errno);
+		throw exc;
 	}
+	if (zcreate_cmd(outFileDescr, partNames) != FD_OK)
+	{
+		(void)::fclose(outFileDescr);
+		(void)::remove(outNameD64.c_str());
+		exc = CFException(CFException::C64ZIP_EXTRACT_FAILED,
+			__LINE__, __FILE__, GetPathname(), "");
+		throw exc;
+	}
+	::fclose(outFileDescr);
 
+	TransitionTo(EXTRACTED);	
 	TRreturn;
 }
 
@@ -763,6 +731,7 @@ void CFZipC64Archive::ProcessFilesInArchive(void) throw (CFException)
 
     
 bool CFZipC64Archive::CheckBelongingAndMergeIfBelongs(const CFArchive& archive)
+	throw (CFException)
 {
 	TRBEGIN("CFZipC64Archive::CheckBelongingAndMergeIfBelongs");
 	TRACE(<< "Candidate " << archive.GetFilename());
@@ -780,7 +749,7 @@ bool CFZipC64Archive::CheckBelongingAndMergeIfBelongs(const CFArchive& archive)
 
 	string pathname = archive.GetPathname();	
 	ASSERT((int)string::npos == -1) //assure that next line works even if there is no /
-	int pos = pathname.find_last_of("/");
+	unsigned int pos = pathname.find_last_of("/");
 	string file = pathname.substr(pos + 1, pathname.length() - pos - 1);
 	
 	ASSERT(file.length() > 1)
@@ -789,7 +758,7 @@ bool CFZipC64Archive::CheckBelongingAndMergeIfBelongs(const CFArchive& archive)
 	if (postfix == m_filename.substr(2, file.length() - 2) + m_extension)
 	{
 		//check for those parts that are in the other archive
-		for (int i = 0; i < 4; i++)
+		for (unsigned int i = 0; i < 4; i++)
 		{
 			if (pMultiPart->TestPartAvailable(i))
 			{
@@ -828,7 +797,7 @@ throw (CFException)
 	string storeArchive = CFArchive::m_filename;
 	for (int i = 1; i < 5; i++)
 	{
-		m_filename[0] = '0' + i;
+		m_filename[0] = (char)('0' + i);
 		//attention: we need all four parts, else there is an exception
 		CFArchive::CopyThisArchiveToDirectory(directory);
 	}
@@ -851,7 +820,7 @@ bool CFPc64Archive::CheckFileIsOfClass(const string& filename)
 	}
 	
 	char puffer[8];
-	in.read(puffer, sizeof(puffer));
+	(void)in.read(puffer, sizeof(puffer));
 	//archive is solely recognized by string pattern
 	if (!strcmp(puffer, "C64File"))
 	{
@@ -861,51 +830,52 @@ bool CFPc64Archive::CheckFileIsOfClass(const string& filename)
 }
 
 
-CFPc64Archive::CFPc64Archive(const string& path, const int blockSize /*= 0*/)
+CFPc64Archive::CFPc64Archive(const string& path, const unsigned int blockSize /*= 0*/)
 	throw (CFException)
-	: m_contentFileByteSize(0), m_pInStream(NULL)
-	, CFC64Archive(path, blockSize)
+	: CFC64Archive(path, blockSize)
+	, m_pInStream(NULL)
+	, m_contentFileByteSize(0)
 {
 	//bit within dir entry set when file is closed
 	const unsigned char closed = 0x80;
 	//bit within dir entry set when file is locked
 	const unsigned char locked = 0x40;
 	//this is the filetype recognized from extension to be put into dir entry
-	unsigned char nFileType = CFDiskLayout::PRG | closed;
+	unsigned char nFileType = CFFileInfoCBM::PRG | closed;
 
 	string extension = m_extension;
 	//make chars lower case for simple comparison
-	for (int i = 0; i < extension.size(); i++)
-		extension[i] = tolower(extension[i]);
+	for (string::size_type i = 0; i < extension.size(); i++)
+		extension[i] = (char)tolower(extension[i]);
 
 	if (extension == ".p00")
 	{
-		nFileType = CFDiskLayout::PRG | closed;
+		nFileType = CFFileInfoCBM::PRG | closed;
 	}
 	else
 	{
 		if (extension == ".s00")
 		{
-			nFileType = CFDiskLayout::SEQ | closed;
+			nFileType = CFFileInfoCBM::SEQ | closed;
 		}
 		else
 		{
 			if (extension == ".u00")
 			{
-				nFileType =  CFDiskLayout::USR | closed;
+				nFileType =  CFFileInfoCBM::USR | closed;
 			}
 			else
 			{
 				if (extension == ".d00")
 				{
 					//DEL files must be locked to appear
-					nFileType = CFDiskLayout::DEL | closed | locked;
+					nFileType = CFFileInfoCBM::DEL | closed | locked;
 				}
 				else
 				{
 					if (extension == ".r00")
 					{
-						nFileType = CFDiskLayout::REL | closed;
+						nFileType = CFFileInfoCBM::REL | closed;
 					}
 				}
 			}
@@ -914,6 +884,7 @@ CFPc64Archive::CFPc64Archive(const string& path, const int blockSize /*= 0*/)
 	//now get the C64filename from the file header and leave file open
 	//for processing
 	m_pInStream = new ifstream(m_pathname.c_str(), ios::in | ios::binary);
+	ASSERT(m_pInStream != NULL);
 	if (!(*m_pInStream))
 	{
 		exc = CFException(CFException::FILE_OPEN_FAILED,
@@ -921,10 +892,10 @@ CFPc64Archive::CFPc64Archive(const string& path, const int blockSize /*= 0*/)
 		throw exc;
 	}
 	//skip 8 bytes of file type pattern
-	m_pInStream->seekg(0x8);
+	(void)m_pInStream->seekg(0x8);
 	char puffer[17];
 	//read 17 bytes c64 filename padded with '\0'
-	m_pInStream->read(puffer, sizeof(puffer));
+	(void)m_pInStream->read(puffer, sizeof(puffer));
 	if (!(*m_pInStream))
 	{
 		m_pInStream->close();
@@ -944,15 +915,14 @@ void CFPc64Archive::Extract(void) throw (CFException)
 	//if already extracted or mother of all archives do no extraction
 	if ((m_state == EXTRACTED) || (m_state == ROOT))//already done
 		TRreturn;
-
-	ASSERT(m_extractCommand.empty())
 			
 	//extract this archive into a new directory to avoid double processing
 	//(found by directory scan)
 	m_workdir = NameAndCreateTmpDir();
 
+	ASSERT(m_pInStream != NULL);
 	ASSERT(m_pInStream->rdbuf()->is_open()); //assure that the stream is opened
-	m_pInStream->seekg(26); //position to start of raw data
+	(void)m_pInStream->seekg(26); //position to start of raw data
 	//open the output file - filename due to CBM filename (maybe we
 	//can keep extensions this way like .lnx)
 	m_contentFilename = m_workdir + "/"
@@ -971,7 +941,7 @@ void CFPc64Archive::Extract(void) throw (CFException)
 	while (m_pInStream->get(ch))
 	{
 		m_contentFileByteSize++; //one more byte written to output file
-		out.put(ch);
+		(void)out.put(ch);
 	}
 
 	if (m_pInStream->bad())
@@ -1010,8 +980,7 @@ void CFPc64Archive::ProcessFilesInArchive(void) throw (CFException)
 	//nonsense.
 	CFArchive *newEntry = GetParentArchivePointer()->
 		CreateSubClassObject(m_contentFilename
-		, (m_contentFileByteSize + CFD64Archive::BLOCKSIZE - 1)
-		/ CFD64Archive::BLOCKSIZE);
+		, (m_contentFileByteSize + BLOCKSIZE_NETTO - 1) / BLOCKSIZE_NETTO);
 	//propagate the CBM fileinfo to brother
 	newEntry->SetFileInfoCBM(GetFileInfoCBM());
 	//add this newly created archive to the others
@@ -1046,13 +1015,14 @@ void CFFinalArchive::ProcessFilesInArchive(void) throw (CFException)
 	//true if a disk image has been created in this call
 	bool bDiskImageFreshlyCreated = false;
 	//this final archive is related to a D64 file located in its parent archive
-	CFDiskImage*& pDiskImage = const_cast<CFArchive *>(GetParentArchivePointer())
+	CFDiskImageD64*& pDiskImage = const_cast<CFArchive *>(GetParentArchivePointer())
 		->GetReferenceToD64EmuImagePointer();
 	//if new diskimage is indicated create a new diskimage
 	if (!pDiskImage)
 	{
 		string d64Name = GetParentArchivePointer()->BuildUniqueEmuImageName();
-		pDiskImage = new CFDiskImage(d64Name);
+		pDiskImage = new CFDiskImageD64(d64Name);
+		ASSERT(pDiskImage != NULL);
 		bDiskImageFreshlyCreated = true;
 	}
 	try {
@@ -1060,16 +1030,19 @@ void CFFinalArchive::ProcessFilesInArchive(void) throw (CFException)
 	{
 		exc = CFException(CFException::FILE_TOO_BIG_FOR_D64
 			,__LINE__, __FILE__, GetPathname()
-			, CFException::IntToString(ReadBlockSize()));
+			, CFException::IntToString((int)ReadBlockSize()));
 		throw exc;
 	}		
 	if (ReadBlockSize() > pDiskImage->GetFreeBlockCountDisk())
 	{
 		//if file won't fit create new emu image
+		//will delete pDiskImage
 		const_cast<CFArchive *>(GetParentArchivePointer())->CloseActualEmuImage();
 		//create a new D64 file
 		string d64Name = GetParentArchivePointer()->BuildUniqueEmuImageName();
-		pDiskImage = new CFDiskImage(d64Name);
+		ASSERT(pDiskImage == NULL);
+		pDiskImage = new CFDiskImageD64(d64Name);
+		ASSERT(pDiskImage != NULL);
 		bDiskImageFreshlyCreated = true;
 	}
 	//insert this final archive into diskimage (keep CBM fileinfo if given)
