@@ -23,23 +23,28 @@
 
 const unsigned int BLOCKSIZE = 256; //overall size of block
 const unsigned int LINKSIZE = 2; //size of the sector link within block (track, sector)
+const unsigned int BLOCKSIZE_NETTO = BLOCKSIZE - LINKSIZE; //size of data in block
 
 
+class CFDiskLayout;
+
 //concept of addressing a block on disk
 //this is done by (track, sector)
 class CFBlockAddress
 {
 public:
-	inline CFBlockAddress(void) : m_track(1), m_sector(0) {}
-	//P1 I: track
-	//P2 I: sector
-	inline CFBlockAddress(const unsigned int track, const unsigned int sector)
-		: m_track(track), m_sector(sector) {}
+	//P1 I: pointer to disk layout to use
+	//P2 I: track
+	//P3 I: sector
+	inline CFBlockAddress(const CFDiskLayout *pLayout
+		, const unsigned char track = 1
+		, const unsigned char sector = 0)
+		: m_pLayout(pLayout), m_track(track), m_sector(sector) {}
 
 public:
 	//move this block to next block (change to next track when track overflow)
 	//R: next block or OUT_OF_BLOCKS when disk end reached
-	CFBlockAddress operator++(void);
+	CFBlockAddress& operator++(void);
 	//copmpare to block addresses
 	//R: true if equal
 	bool operator==(const CFBlockAddress& a) const;
@@ -57,21 +62,240 @@ public:
 	bool IsValidLink(void) const;
 
 public:
-	unsigned int m_track;
-	unsigned int m_sector;
+	//this class needs disk layout info - there it is
+	const CFDiskLayout *m_pLayout;
+	unsigned char m_track;
+	unsigned char m_sector;
 };
-const CFBlockAddress OUT_OF_BLOCKS(255, 255); //out of disk indicator
+const unsigned char TRACK_INVALID(255);
+const unsigned char SECTOR_INVALID(255);
+const CFBlockAddress OUT_OF_BLOCKS(NULL, TRACK_INVALID, SECTOR_INVALID); //out of disk indicator
 
 
 //concept of capsulating the disk geometry
-//right now for 1541 disks only
 class CFDiskLayout
 {
 public:
-	//these are the filetypes for directory entries with the real values
-	typedef enum { DEL = 0, SEQ = 1, PRG = 2, USR = 3, REL = 4, UNKNOWN }
-		tFiletype;
+	//transfer sector count per track into private vector
+	//initialize all vectors
+	//parameters matched to members directly (look there)
+	inline CFDiskLayout(const unsigned char tracks
+		, const unsigned char sectorPerTrack[]
+		, const CFBlockAddress& controlBlock
+		, const CFBlockAddress& firstDirBlock
+		, const char fileterm
+		, const unsigned char lastDirBlockLength)
+		: m_tracks(tracks)
+		, m_fileTermChar(fileterm)
+		, m_lastDirBlockLength(lastDirBlockLength)
+		, m_sectorPerTrack(sectorPerTrack)
+		, m_controlBlock(controlBlock)
+		, m_firstDirBlock(firstDirBlock)
+		{
+			//assign this layout to block addresses
+			m_controlBlock.m_pLayout = this;
+			m_firstDirBlock.m_pLayout = this;
+		}
 
+public:
+	//R: number of tracks of disk
+	inline unsigned char GetTracks(void) const { return m_tracks; }
+	//P1 I: number of tracks of disk to set
+	inline void SetTracks(unsigned char nTracks) { m_tracks = nTracks; }
+	//R: termination char for directory name entry ($a0 for 1541)
+	inline char GetFileTermChar(void) const { return m_fileTermChar; }
+	//R: block address of control block containing BAM
+	inline  const CFBlockAddress& GetControlBlockAddr(void) const { return m_controlBlock; }
+	//R: length of the last directory block which should always be const
+	inline unsigned char GetLastDirBlockLength(void) const { return m_lastDirBlockLength; }
+	//R: block address of the first directory block
+	inline const CFBlockAddress& GetFirstDirBlockAddr(void) const { return m_firstDirBlock; }
+	//get the number of sectors of a particular track
+	//P1 I: track index - attention: counting starts with 1 (not 0)
+	//R: number of sectors for this track
+	unsigned char operator[](const unsigned char track) const;
+
+private:
+	unsigned char m_tracks; //number of tracks on disk
+	//character that ends an filename within a directory entry name
+	char m_fileTermChar;
+	//length of the last directory block which should always be const
+	unsigned char m_lastDirBlockLength;
+	//sector per track table
+	const unsigned char *m_sectorPerTrack;
+	//block address of the control block containing BAM
+	CFBlockAddress m_controlBlock;
+	//block address of the first directory block
+	CFBlockAddress m_firstDirBlock;
+};
+
+
+//concept of a directory entry within disk directory
+class CFDirectoryEntry
+{
+public:
+#ifdef _MSC_VER
+#pragma pack(push, l1)
+#pragma pack(1)
+#endif
+	//memory layout of a directory entry
+	typedef struct
+	{
+		unsigned char notused1[2];
+		union //we need access to bitfield components and complete byte
+		{
+			unsigned char fileInfoByte; //access complete byte
+			struct
+			{
+				unsigned char filetype : 3; //compare tFileType
+				unsigned char notused  : 3;
+				unsigned char locked   : 1; //true means locked against deletion
+				unsigned char closed   : 1; //true means has been properly closed
+			} fileInfo;
+		};
+		unsigned char dataTrack; 
+		unsigned char dataSector; //link to first data block
+		char name[16];
+		unsigned char notused3[7]; //usage only for relative files
+		unsigned char newTrack;
+		unsigned char newSector; //normally not used
+		unsigned char lengthLow;
+		unsigned char lengthHigh; //block length of entry
+	} tDirEntry;
+#ifdef _MSC_VER
+#pragma pack(pop, l1)
+#endif
+
+public:
+	//construct from single values
+	//member variables accordingly set to parameters
+	CFDirectoryEntry(const string& filename, const unsigned int blockSize
+		, const CFBlockAddress& firstBlock
+		, const CFFileInfoCBM::tFiletype filetype
+		, const bool locked = false, const bool closed = true);
+	//construct from a physical directory entry from disk
+	//P1 is needed to generate a valid CFDiskBlockAddress m_firstBlockAddr
+	//P1 I: disk layout to use
+	CFDirectoryEntry(const CFDiskLayout *pLayout
+		, const tDirEntry& dirEntry);
+	virtual ~CFDirectoryEntry() {}
+
+public:
+	//R: CBM filename
+	inline const CFCbmFilename& GetFilename(void) const
+		{ return m_filename; }
+	//R: block size of entry
+	inline unsigned int GetSize(void) const
+		{ return m_size; }
+	//R: filetype (PRG, SEQ, etc.)
+	inline CFFileInfoCBM::tFiletype GetFiletype(void) const
+		{ return m_filetype; }
+	//R: block address of the first data block
+	inline const CFBlockAddress& GetFirstFileDataBlockAddr(void) const
+		{ return m_firstBlockAddr; }
+	//conversion operator to produce a physical layout of this logical entry
+	//R: physical layout (which may directly be put into dir block)
+	operator CFDirectoryEntry::tDirEntry&(void) const; 
+
+private:
+	bool m_locked; //true indicates locking against deletion
+	bool m_closed; //true indicates properly closed file
+	CFCbmFilename m_filename; //CBM filename (planar, size limited)
+	unsigned short m_size; //blocksize
+	CFFileInfoCBM::tFiletype m_filetype; //filetype (PRG, SEQ, etc.)
+	CFBlockAddress m_firstBlockAddr; //first data block address of file
+};
+
+
+//TODO: check if superclass should be more general for D81 and others
+
+//we have to pass a D64 geometry so specialize here
+class CFDirectoryEntryD64 : public CFDirectoryEntry
+{
+public:
+	//construct from single values
+	//member variables accordingly set to parameters
+	CFDirectoryEntryD64(const string& filename, const unsigned int blockSize
+		, const CFBlockAddress& firstBlock
+		, const CFFileInfoCBM::tFiletype filetype
+		, const bool locked = false, const bool closed = true)
+		: CFDirectoryEntry(filename, blockSize, firstBlock, filetype, locked, closed)
+	{}
+	//construct from a physical directory entry from disk
+	CFDirectoryEntryD64(const tDirEntry& dirEntry);
+};
+
+//we have to pass a D64Ex geometry so specialize here
+class  CFDirectoryEntryD64Ex : public CFDirectoryEntry
+{
+public:
+	//construct from single values
+	//member variables accordingly set to parameters
+	CFDirectoryEntryD64Ex(const string& filename, const unsigned int blockSize
+		, const CFBlockAddress& firstBlock
+		, const CFFileInfoCBM::tFiletype filetype
+		, const bool locked = false, const bool closed = true)
+		: CFDirectoryEntry(filename, blockSize, firstBlock, filetype, locked, closed)
+	{}
+	//construct from a physical directory entry from disk
+	CFDirectoryEntryD64Ex(const tDirEntry& dirEntry);
+};
+
+
+//concept of a disk block containing raw data
+//structured into header (linkage) and real data
+class CFDiskBlock
+{
+public:
+	//P1 I: disk layout to use
+	CFDiskBlock(const CFDiskLayout *pLayout);
+public:
+	//read a byte of raw data from block
+	//P1 I: byte index
+	//R: reference to read character
+	unsigned char& operator[](const unsigned int index);
+	//R: pointer to the raw data
+	inline unsigned char *GetBlockContentPointer(void) { return m_content; }
+	//R: block linkage (header part of raw data)
+	inline CFBlockAddress ReadSectorLink(void) const
+		{ return CFBlockAddress(m_pLayout, m_content[0], m_content[1]); }
+	//overwrite the linkage (header part of raw data)
+	//P1 I: block address to write
+	inline void WriteSectorLink(const CFBlockAddress& link)
+		{ m_content[0] = link.m_track; m_content[1] = link.m_sector; }
+	//read data from P1 as long not EOF and still space in block
+	//put data starting with beginning of raw data (index 0)
+	//P1 IO: stream to read data from
+	//R: number of bytes got
+	unsigned int GetHeaderAndDataFromStream(istream& in) throw (CFException);
+	//read data from P1 as long not EOF and still space in block
+	//put data starting with beginning of real data (skip header = linkage)
+	//P1 IO: stream to read data from
+	//R: number of bytes got
+	unsigned int GetRealDataOnlyFromStream(istream& in) throw (CFException);
+	//write raw data (header + real data) to P1
+	//P1 IO: stream to write data to
+	//R: number of bytes put - always BLOCKSIZE (else exception)
+	unsigned int PutHeaderAndDataToStream(ostream& out) const throw (CFException);
+	//write real data only to stream P1.
+	//if block is a last data block (link track == 0) write only data
+	//indicated else write BLOCKSIZE byte
+	//P1 IO: stream to write data to
+	//R: number of bytes put
+	unsigned int PutRealDataOnlyToStream(ostream& out) const throw (CFException);
+
+private:
+	//this class needs disk layout info - there it is
+	const CFDiskLayout *m_pLayout;
+	//raw data of block
+	unsigned char m_content[BLOCKSIZE];
+};
+
+
+//concept of the control block which contains management info like BAM
+class CFControlBlock
+{
+public:
 //it seems that g++ has default alignment 1 byte (ASSERTS work)
 #ifdef _MSC_VER
 #pragma pack(push, l1)
@@ -91,175 +315,129 @@ public:
 		unsigned char linkFirstDirBlockSector; //links to first dir block
 		unsigned char formatID; //should be 0x41 for 1541
 		unsigned char unused1;
-		tTrackBam Bam[35]; //BAM for the complete disk
+		tTrackBam Bam[35]; //BAM for tracks 1-35
 		char diskName[16]; //this is the name passed while formating
 		unsigned char delimiter1[2]; //should be both 0xa0
 		unsigned char diskId[2]; //this is the disk ID passed while formating
 		unsigned char delimiter2; //should be 0xa0
 		unsigned char format[2]; //should be 0x32 0x41
 		unsigned char delimiter3[4]; //should be 0xa0
-		unsigned char unused2[85];
+		unsigned char unused2[21];
+		tTrackBam BamEx[5]; //BAM for tracks 36-40
+		unsigned char unused3[44];
 	} tControlBlock;
-
-	//memory layout of a directory entry
-	typedef struct
-	{
-		unsigned char notused1[2];
-		unsigned char filetype : 3; //compare tFileType
-		unsigned char notused2 : 3;
-		unsigned char locked   : 1; //true means locked against deletion
-		unsigned char closed   : 1; //true means has been properly closed
-		unsigned char dataTrack; 
-		unsigned char dataSector; //link to first data block
-		char name[16];
-		unsigned char notused3[7]; //usage only for relative files
-		unsigned char newTrack;
-		unsigned char newSector; //normally not used
-		unsigned char lengthLow;
-		unsigned char lengthHigh; //block length of entry
-	} tDirEntry;
 #ifdef _MSC_VER
 #pragma pack(pop, l1)
 #endif
 
 public:
-	//transfer sector count per track into private vector
-	//initialize all vectors
-	//parameters matched to members directly (look there)
-	inline CFDiskLayout(const unsigned int tracks
-		, const unsigned int sectorPerTrack[]
-		, const CFBlockAddress& controlBlock
-		, const CFBlockAddress& firstDirBlock
-		, const char fileterm
-		, const unsigned char lastDirBlockLength)
-		: m_tracks(tracks), m_controlBlock(controlBlock)
-		, m_firstDirBlock(firstDirBlock)
-		, m_fileTermChar(fileterm)
-		, m_lastDirBlockLength(lastDirBlockLength)
-		{
-			//move the sector information into the vector
-			for (int i = 0; i < m_tracks; i++)
-				m_sectorPerTrack.push_back(sectorPerTrack[i]);
-		}
-
+	//construct virgin control block
+	//P1 I: disk layout to use
+	//P2 I: disk name as passed for formatting
+	//P3 I: disk ID as passed for formatting
+	CFControlBlock(const CFDiskLayout *pLayout
+		, const CFCbmFilename& diskname = CFCbmFilename()
+		, const char formatID[2] = "64");
 public:
-	//R: number of tracks of disk
-	inline unsigned int GetTracks(void) const { return m_tracks; }
-	//R: termination char for directory name entry ($a0 for 1541)
-	inline char GetFileTermChar(void) const { return m_fileTermChar; }
-	//R: block address of control block containing BAM
-	inline  const CFBlockAddress& GetControlBlockAddr(void) const { return m_controlBlock; }
-	//R: length of the last directory block which should always be const
-	inline unsigned char GetLastDirBlockLength(void) const { return m_lastDirBlockLength; }
-	//R: block address of the first directory block
-	inline const CFBlockAddress& GetFirstDirBlockAddr(void) const { return m_firstDirBlock; }
-	//get the number of sectors of a particular track
-	//P1 I: track index - attention: counting starts with 1 (not 0)
-	//R: number of sectors for this track
-	inline unsigned int operator[](const unsigned int track)
-		{ return m_sectorPerTrack[track - 1]; }
+	//attach control block: store it (p1) in local memory
+	//check consistency - correct if necessary (especially free block count)
+	//P1 I: content of control block
+	void AttachControlBlock(const CFDiskBlock& controlBlock) throw (CFException);
+	//test the allocation of a block P1
+	//P1 I: addresses block to test
+	//R: true if allocated
+	bool TestBlockAllocation(const CFBlockAddress& addr) const;
+	//allocate block P1 in BAM. must be free before calling this method.
+	//P1 I: addresses block to test
+	void AllocBlock(const CFBlockAddress& addr);
+	//free block P1 in BAM. must be allocated before calling this method.
+	//P1 I: addresses block to test
+	void FreeBlock(const CFBlockAddress& addr);
+	//R: the free block count of the whole disk
+	unsigned int GetFreeBlockCountDisk(void) const;
+	//search the directory track for a free block and alloc it if found
+	//control block is not considered.
+	//R: OUT_OF_BLOCKS if not found else address of found block
+	CFBlockAddress FindAndAllocFreeDirBlock(void);
+	//peek for a free block ouside directory track - do not alloc!
+	//R: OUT_OF_BLOCKS if not found else address of found block
+	CFBlockAddress FindWithoutAllocFirstFreeFileBlock(void) const;
+	//find a free block outside directory track and alloc it if found
+	//R: OUT_OF_BLOCKS if not found else address of found block
+	CFBlockAddress FindAndAllocFirstFreeFileBlock(void);
+	//write the control block into a block 
+	//P1 O: control block written
+	void DetachControlBlock(CFDiskBlock& out) const;
+private:
+	//provide a virgin initialized control block
+	//virgin BAM and control block first dir block allocated
+	//P1 I: disk name as passed for formatting
+	//P2 I: disk ID as passed for formatting
+	void InitVirginControlBlock(const CFCbmFilename& diskname
+		, const char diskId[2]);
 
 private:
-	unsigned int m_tracks; //number of tracks on disk
-	//character that ends an filename within a directory entry name
-	char m_fileTermChar;
-	//length of the last directory block which should always be const
-	unsigned char m_lastDirBlockLength;
-	//sector per track table
-	vector<unsigned int> m_sectorPerTrack;
-	//block address of the control block containing BAM
-	CFBlockAddress m_controlBlock;
-	//block address of the first directory block
-	CFBlockAddress m_firstDirBlock;
+	//this class needs disk layout info - there it is
+	const CFDiskLayout *m_pLayout;
+
+protected:
+	//raw data of control block
+	tControlBlock m_controlBlock;
 };
 
 
-//concept of a directory entry within disk directory
-class CFDirectoryEntry
+//CFControlBlock is designed for D64 so we do not have to change
+//anything here
+//TODO: check if superclass should be more general for D81 and others
+typedef CFControlBlock CFControlBlockD64;
+
+
+//concept of an extended D64 control block (extended BAM for tracks 36-40)
+class CFControlBlockD64Ex : public CFControlBlockD64
 {
 public:
-	CFDirectoryEntry(void);
-	//construct from single values
-	//member variables accordingly set to parameters
-	CFDirectoryEntry(const string& filename, const unsigned int blockSize
-		, const CFBlockAddress& firstBlock
-		, const CFDiskLayout::tFiletype filetype
-		, const bool locked = false, const bool closed = true);
-	//construct from a physical directory entry from disk
-	CFDirectoryEntry(const CFDiskLayout::tDirEntry& dirEntry);
-
+	//construct virgin control block
+	//P1 I: disk layout to use
+	//P2 I: disk name as passed for formatting
+	//P3 I: disk ID as passed for formatting
+	CFControlBlockD64Ex(const CFDiskLayout *pLayout
+		, const CFCbmFilename& diskname = CFCbmFilename()
+		, const char formatID[2] = "64");
 public:
-	//R: CBM filename
-	inline const CFCbmFilename& GetFilename(void) const
-		{ return m_filename; }
-	//R: block size of entry
-	inline unsigned int GetSize(void) const
-		{ return m_size; }
-	//R: filetype (PRG, SEQ, etc.)
-	inline CFDiskLayout::tFiletype GetFiletype(void) const
-		{ return m_filetype; }
-	//R: block address of the first data block
-	inline const CFBlockAddress& GetFirstFileDataBlockAddr(void) const
-		{ return m_firstBlockAddr; }
-	//conversion operator to prduce a physical layout of this logical entry
-	//R: physical layout (which may directly be put into dir block)
-	operator CFDiskLayout::tDirEntry&(void) const; 
+	//attach control block: store it (p1) in local memory
+	//check consistency - correct if necessary (especially free block count)
+	//P1 I: content of control block
+	void AttachControlBlock(const CFDiskBlock& controlBlock) throw (CFException);
+	//test the allocation of a block P1
+	//P1 I: addresses block to test
+	//R: true if allocated
+	bool TestBlockAllocation(const CFBlockAddress& addr) const;
+	//allocate block P1 in BAM. must be free before calling this method.
+	//P1 I: addresses block to test
+	void AllocBlock(const CFBlockAddress& addr);
+	//free block P1 in BAM. must be allocated before calling this method.
+	//P1 I: addresses block to test
+	void FreeBlock(const CFBlockAddress& addr);
+	//R: the free block count of the whole disk
+	unsigned int GetFreeBlockCountDisk(void) const;
+	//peek for a free block ouside directory track - do not alloc!
+	//R: OUT_OF_BLOCKS if not found else address of found block
+	CFBlockAddress FindWithoutAllocFirstFreeFileBlock(void) const;
+	//find a free block outside directory track and alloc it if found
+	//R: OUT_OF_BLOCKS if not found else address of found block
+	CFBlockAddress FindAndAllocFirstFreeFileBlock(void);
 
 private:
-	bool m_locked; //true indicates locking against deletion
-	bool m_closed; //true indicates properly closed file
-	CFCbmFilename m_filename; //CBM filename (planar, size limited)
-	unsigned short m_size; //blocksize
-	CFDiskLayout::tFiletype m_filetype; //filetype (PRG, SEQ, etc.)
-	CFBlockAddress m_firstBlockAddr; //first data block address of file
-};
-
-
-//concept of a disk block containing raw data
-//structured into header (linkage) and real data
-class CFDiskBlock
-{
-public:
-	CFDiskBlock(void);
-
-public:
-	//read a byte of raw data from block
-	//P1 I: byte index
-	//R: reference to read character
-	unsigned char& operator[](const unsigned int index);
-	//R: pointer to the raw data
-	inline unsigned char *GetBlockContentPointer(void) { return m_content; }
-	//R: block linkage (header part of raw data)
-	inline CFBlockAddress ReadSectorLink(void) const
-		{ return CFBlockAddress(m_content[0], m_content[1]); }
-	//overwrite the linkage (header part of raw data)
-	//P1 I: block address to write
-	inline void WriteSectorLink(const CFBlockAddress& link)
-		{ m_content[0] = link.m_track; m_content[1] = link.m_sector; }
-	//read data from P1 as long not EOF and still space in block
-	//put data starting with beginning of raw data (index 0)
-	//P1 IO: stream to read data from
-	//R: number of bytes got
-	unsigned int GetHeaderAndDataFromStream(istream& in) throw (CFException);
-	//read data from P1 as long not EOF and still space in block
-	//put data starting with beginning of real data (skip header = linkage)
-	//P1 IO: stream to read data from
-	//R: number of bytes got
-	unsigned int GetRealDataOnlyFromStream(istream& in);
-	//write raw data (header + real data) to P1
-	//P1 IO: stream to write data to
-	//R: number of bytes put - always BLOCKSIZE (else exception)
-	unsigned int PutHeaderAndDataToStream(ostream& out) const throw (CFException);
-	//write real data only to stream P1.
-	//if block is a last data block (link track == 0) write only data
-	//indicated else write BLOCKSIZE byte
-	//P1 IO: stream to write data to
-	//R: number of bytes put
-	unsigned int PutRealDataOnlyToStream(ostream& out) const throw (CFException);
+	//provide a virgin initialized control block
+	//virgin BAM and control block first dir block allocated
+	//P1 I: disk name as passed for formatting
+	//P2 I: disk ID as passed for formatting
+	void InitVirginControlBlock(const CFCbmFilename& diskname
+		, const char diskId[2]);
 
 private:
-	//raw data of block
-	unsigned char m_content[BLOCKSIZE];
+	//this class needs disk layout info - there it is
+	const CFDiskLayout *m_pLayout;
 };
 
 
@@ -267,26 +445,23 @@ const string MEMORY_DISKIMAGE_NAME("Memory diskimage"); //indicates memory image
 //concept of a disk image
 //image may be attached to a file. after that all processing in memory
 //until WriteDiskMemoryToFilesystem().
+//TP1 I: directory entry class
+//TP2 I: control block class
+template <class TDirectoryEntry, class TControlBlock>
 class CFDiskImage
 {
-//control block is part of image - make it inner class
-class CFControlBlock;
-//control block may access private members of disk image
-friend class CFControlBlock;
-
 public:
 	//allocate memory for disk (alls disk blocks)
 	//prepare directory
-	//P1 I: host filesystem name of disk image
-	//P2 I: disk name as passed for formatting
-	//P3 I: disk ID as passed for formatting
-	CFDiskImage(const string& filename = MEMORY_DISKIMAGE_NAME
+	//P1 I: pointer to disk layout to use
+	//P2 I: host filesystem name of disk image
+	//P3 I: disk name as passed for formatting
+	//P4 I: disk ID as passed for formatting
+	CFDiskImage(const CFDiskLayout *pLayout
+		, const string& filename = MEMORY_DISKIMAGE_NAME
 		, const CFCbmFilename& diskname = CFCbmFilename()
 		, const char diskId[2] = "64");
-private:
-	//just to calculate a few offsets in CFControlBlock::CFControlBlock
-	//need constructor which does no alloc
-	CFDiskImage(char *) {}
+	virtual ~CFDiskImage() {}
 
 public:
 	//extract all files (except unextractable liek REL) on disk into
@@ -296,7 +471,7 @@ public:
 	//keep track of all extracted files in m_extractedFiles
 	//P1 I: directory to extract to
 	//R: files remaining not extracted (e.g. REL files)
-	bool ExtractAllFiles(const string& dirname);
+	bool ExtractAllFiles(const string& dirname)	throw (CFException);
 	//R: vector with info about extracted files (needed for transfer into D64)
 	inline const vector<CFExtractedFileInfo>& GetVectorOfExtractedFiles(void) const
 		{ return m_extractedFiles; }
@@ -313,7 +488,7 @@ public:
 	//P1 I: disk image to attach within host filesystem
 	void Attach(const string& filename) throw (CFException);
 	//R: reference to parsed directory
-	inline const vector<CFDirectoryEntry>& GetDirectory(void) const
+	inline const vector<TDirectoryEntry>& GetDirectory(void) const
 		{ return m_directory; }
 	//insert a file into disk image (in memory)
 	//this includes writing of a directory entry slot on disk
@@ -328,7 +503,7 @@ public:
 		, const CFFileInfoCBM& tapeFileInfo = CFFileInfoCBM()) throw (CFException);
 	//flush the memory image to host filesystem file
 	//write first to a temporary file and rename afterwards (transaction)
-	void WriteDiskMemoryToFilesystem(void) const throw (CFException); 
+	void WriteDiskMemoryToFilesystem(void) throw (CFException); 
 	//R: the free block count of the whole disk
 	inline unsigned int GetFreeBlockCountDisk(void) const
 	{ return m_control.GetFreeBlockCountDisk(); }
@@ -341,7 +516,7 @@ private:
 	//P2 I: directory to extract to
 	//P3 O: additional file info from directory entry
 	//P4 I: host filesystem filename ("" means not set)
-	void ExtractFile(const CFDirectoryEntry& entry
+	void ExtractFile(const TDirectoryEntry& entry
 		, const string& dirname
 		, CFExtractedFileInfo& extractInfo
 		, const string& filename = "") const throw (CFException);
@@ -349,71 +524,54 @@ private:
 	//if found write P1 into it. do alloc of dir block if needed.
 	//add entry to parsed directory m_directory
 	//P1 I: directory entry to be put into directory
-	void InsertNewDirectoryEntryWithBlockAlloc(const CFDirectoryEntry& entry);
+	void InsertNewDirectoryEntryWithBlockAlloc(const TDirectoryEntry& entry)
+		throw (CFException);
 
 private:
 	//fileinfo of all extracted files of this disk
 	vector<CFExtractedFileInfo> m_extractedFiles;
+	//error info for all disk blocks (optional)
+	vector<unsigned char> m_errorInfo;
 	typedef vector<CFDiskBlock> tSectorsInTrack;
 	vector<tSectorsInTrack> m_diskBlock; //all blocks of the disk
-	vector<CFDirectoryEntry> m_directory; //logical representation of disk dir
+	vector<TDirectoryEntry> m_directory; //logical representation of disk dir
 	string m_filename; //host filesystem filename of diskimage
 
-public:
-	//disk geometry of disk
-	static CFDiskLayout m_geom;
-
 private:
-	//concept of the control block which contains management info like BAM
-	class CFControlBlock
-	{
-	public:
-		//construct virgin control block
-		//P1 I: disk name as passed for formatting
-		//P2 I: disk ID as passed for formatting
-		CFControlBlock(const CFCbmFilename& diskname = CFCbmFilename()
-			, const char formatID[2] = "64");
-	public:
-		//attach control block from memory disk
-		//check consistency - correct if necessary (especially free block count)
-		void AttachControlBlock(void) throw (CFException);
-		//test the allocation of a block P1
-		//P1 I: addresses block to test
-		//R: true if allocated
-		bool TestBlockAllocation(const CFBlockAddress& addr) const;
-		//allocate block P1 in BAM. must be free before calling this method.
-		//P1 I: addresses block to test
-		void AllocBlock(const CFBlockAddress& addr);
-		//free block P1 in BAM. must be allocated before calling this method.
-		//P1 I: addresses block to test
-		void FreeBlock(const CFBlockAddress& addr);
-		//R: the free block count of the whole disk
-		unsigned int GetFreeBlockCountDisk(void) const;
-		//search the directory track for a free block and alloc it if found
-		//control block is not considered.
-		//R: OUT_OF_BLOCKS if not found else address of found block
-		CFBlockAddress FindAndAllocFreeDirBlock(void);
-		//peek for a free block ouside directory track - do not alloc!
-		//R: OUT_OF_BLOCKS if not found else address of found block
-		CFBlockAddress FindWithoutAllocFirstFreeFileBlock(void) const;
-		//find a free block outside directory track and alloc it if found
-		//R: OUT_OF_BLOCKS if not found else address of found block
-		CFBlockAddress FindAndAllocFirstFreeFileBlock(void);
-		//write the control block back to memory image
-		void WriteToMemoryDisk(void) const;
-	private:
-		//provide a virgin initialized control block
-		//virgin BAM and control block first dir block allocated
-		//P1 I: disk name as passed for formatting
-		//P2 I: disk ID as passed for formatting
-		void InitVirginControlBlock(const CFCbmFilename& diskname
-			, const char diskId[2]);
-	private:
-		//pointer to enclosing class (calculated while construction!)
-		CFDiskImage* m_pEnclosing;
-		//raw data of control block
-		CFDiskLayout::tControlBlock m_controlBlock;
-	} m_control;
+	//this class needs disk layout info - there it is
+	const CFDiskLayout *m_pLayout;
+	TControlBlock m_control;
 };
 
+
+//concept of a D64 image
+class CFDiskImageD64 : public CFDiskImage<CFDirectoryEntryD64, CFControlBlockD64>
+{
+public:
+	//allocate memory for disk (all disk blocks)
+	//prepare directory
+	//P1 I: host filesystem name of disk image
+	//P2 I: disk name as passed for formatting
+	//P3 I: disk ID as passed for formatting
+	CFDiskImageD64(const string& filename = MEMORY_DISKIMAGE_NAME
+		, const CFCbmFilename& diskname = CFCbmFilename()
+		, const char diskId[2] = "64");
+};
+
+
+//concept of an extended D64 image (tracks 1-40)
+class CFDiskImageD64Ex : public CFDiskImage<CFDirectoryEntryD64Ex, CFControlBlockD64Ex>
+{
+public:
+	//allocate memory for disk (all disk blocks)
+	//prepare directory
+	//P1 I: host filesystem name of disk image
+	//P2 I: disk name as passed for formatting
+	//P3 I: disk ID as passed for formatting
+	CFDiskImageD64Ex(const string& filename = MEMORY_DISKIMAGE_NAME
+		, const CFCbmFilename& diskname = CFCbmFilename()
+		, const char diskId[2] = "64");
+};
+
+
 #endif
