@@ -400,8 +400,13 @@ get_header (fp, hdr)
       return FALSE;		/* finish */
     }
 
+#ifdef TD64_MODIFIED
+  if ((int)fread (data + I_HEADER_CHECKSUM,
+		  sizeof (char), header_size - 1, fp) < header_size - 1)
+#else
   if (fread (data + I_HEADER_CHECKSUM,
 		  sizeof (char), header_size - 1, fp) < header_size - 1)
+#endif
     {
       fatal_error ("Invalid header (LHarc file ?)");
       return FALSE;		/* finish */
@@ -502,7 +507,11 @@ get_header (fp, hdr)
       {
 	if (hdr->header_level != 2 &&
 	    ((data + LZHEADER_STRAGE - get_ptr < header_size) ||
+#ifdef TD64_MODIFIED
+	     (int)fread(get_ptr, sizeof(char), header_size, fp) < header_size))
+#else
 	     fread(get_ptr, sizeof(char), header_size, fp) < header_size))
+#endif
 	  {
 	    fatal_error ("Invalid header (LHa file ?)");
 	    return FALSE;
@@ -539,7 +548,11 @@ get_header (fp, hdr)
 	  if (hdr->extend_type == EXTEND_MSDOS ||
 	      hdr->extend_type == EXTEND_HUMAN ||
 	      hdr->extend_type == EXTEND_GENERIC)
+#ifdef TD64_MODIFIED
+	    hdr->attribute = (unsigned char)get_word();
+#else
 	    hdr->attribute = get_word();
+#endif
 	  break;
 	case 0x50:
 	  /*
